unit udemandaconrespuesta;

(**  20150804 Felipe y Lorena ***)

interface

uses
  Classes,
  uSimplex,
  xmatdefs,
  dateutils,
  fddp,
  SysUtils;

resourcestring
  rsdemandaConRespuesta = 'Demanda con Respuesta';


type
  arreglos = array [1..168] of NReal;

  TdemandaConRespuesta = class // forward declaration

    //  { TdemandaConRespuesta }

  public
    dtUltimaLLamda: TDateTime;

    E_ini: NReal; // valor inicial al inicio de la primer hora
    E_finDelPaso: NReal;

    Econs: NReal;// carga consumida luego de cada hora deadline

    E: NReal;


    E_max, Emin: NReal;
    //Energía máxima almacenable del equipo y la energía mínima objetivo según vector_deadline
    PMax_Gen, PMax_Dem: NReal;


    ren_Gen, ren_Dem: NReal;
    //rendimiento como generador (inyecta a la red) y como Demanda (consume)


    ro: NReal; // Ek+1 = ro*Ek+ ren_Dem * PDem* - 1/ren_Gen * PGen

    vector_operabilidad, vector_deadline: arreglos;

    peso_InicioHora, peso_FinHora: NReal;
    // de 0 a 1 el tiempo desde el inicio de la hora en que hace la consulta.
    // suponemos que cada controlador se le contesta una vez por hora a lo sumo.

    constructor Create(E_ini, Econs, E_max, Emin, PMax_Gen, PMax_Dem,
      ren_Gen, ren_Dem, ro: NReal;
      vector_operabilidad, vector_deadline: arreglos);

    procedure CDR_MW(
       var DCR_MW: TDAOFNReal; // Acumula la demanda en MW hora a hora.
       var Energias: TDAOfNReal; // Acumla la energía almacenada
       cmg_ve_ant, cmg_ve_: TDAofNReal;
    // INformación de costos marginales hora a hora.
       dtInicioDelTramo: TDateTime);
  end;

  TPoblacion = class( TList )
    Nombre: string;

    Energias: TDAOFNReal;
    Potencias: TDAOfNReal;

    constructor Create( Nombre: String; nHorasTramo: integer );
    procedure CDR_MW(
       var DCR_MW: TDAOFNReal; // Acumula la demanda en MW hora a hora.
       cmg_ve_ant, cmg_ve_: TDAofNReal;
    // INformación de costos marginales hora a hora.
       dtInicioDelTramo: TDateTime; flg_ActualizarEIni: boolean );
  end;

var
  Poblaciones: TList;
  dt_UltimaLLamada: TDateTime;
  cmg_ANTERIOR: TDAOfNReal;




function CrearCalefones(Poblacion: integer; HoraUsoInicio, HoraUsoFin: integer;
  Volumen_lt: NReal; // 60lt, 90lt
  PMax_kW: NReal; renPerdidasPorHora: NREal; // ro
  nHorasTramo: integer // horas del tramo a simular EJ. 72 horas
  ): TPoblacion;

function CrearFlotaAutosElectricos(Poblacion: integer;
  HoraUsoInicio, HoraUsoFin: integer; Capacidad: NReal; // 12,16..20 kWh
  PMax_kW_carga, PMax_kW_descarga: NReal;
  //potencia máxima de demanda o inyección a la red
  renPerdidasPorHora: NREal; // ro
  nHorasTramo: integer // horas del tramo a simular EJ. 72 horas
): TPoblacion;


procedure CDR_MW(var DCR_MW: TDAOFNReal; // Retorna la demanda en MW hora a hora.
  cmg_HoraAnt, cmg_ve: TDAofNReal;
  // INformación de costos marginales hora a hora.
  dtInicioDelTramo: TDateTime; flg_ImponerCMGsDecreciente: boolean);



implementation

var
  fuenteRuido: Tf_ddpUniformeRand3;


function random: NReal;
begin
  Result := FuenteRuido.rnd;
end;



constructor TPoblacion.Create( Nombre: String; nHorasTramo: integer );
begin
  inherited Create;
  self.Nombre:= nombre;
  setlength( Energias, nHorasTramo );
  setlength( Potencias, nHorasTramo );
end;

procedure TPoblacion.CDR_MW(
   var DCR_MW: TDAOFNReal; // Acumula la demanda en MW hora a hora.
   cmg_ve_ant, cmg_ve_: TDAofNReal;
// INformación de costos marginales hora a hora.
   dtInicioDelTramo: TDateTime; flg_ActualizarEIni: boolean );

var
  k: integer;
  a: TdemandaConRespuesta;

begin
  vclear( Energias );
  for k := 0 to Count - 1 do
  begin
    a := items[k];
    if flg_ActualizarEini then
      a.E_ini := a.E_finDelPaso;
    a.CDR_MW(DCR_MW, Energias, cmg_ve_ant, cmg_ve_, dtInicioDelTramo);
  end;
end;

procedure CDR_MW(var DCR_MW: TDAOFNReal; // Retorna la demanda en MW hora a hora.
  cmg_HoraAnt, cmg_ve: TDAofNReal;
  // INformación de costos marginales hora a hora.
  dtInicioDelTramo: TDateTime; flg_ImponerCMGsDecreciente: boolean);

var
  k, j: integer;
  flg_ActualizarEini: boolean;
  aPoblacion: TPoblacion;

begin
  if flg_ImponerCMGsDecreciente then
  begin
    for k := 0 to high(cmg_ve) do
    begin
      cmg_ve[k] := 100;
      cmg_HoraAnt[k] := 100;
    end;
  end;

  if (dtInicioDelTramo > dt_UltimaLLamada) then
  begin
    flg_ActualizarEini := True;
    dt_UltimaLLamada := dtInicioDelTramo;
    fuenteRuido.Reiniciar(trunc(dtInicioDelTramo * 24) + 31);
  end
  else
    flg_ActualizarEini := False;

  vclear(DCR_MW);

  for j:= 0 to Poblaciones.count -1 do
  begin
    aPoblacion:= Poblaciones[j];
    aPoblacion.CDR_MW( DCR_MW, cmg_HoraAnt, cmg_ve, dtINicioDelTramo, flg_ActualizarEini );
  end;

end;

function CrearFlotaAutosElectricos(Poblacion: integer;
  HoraUsoInicio,// Hora a partir de la cual uso  el auto
  HoraUsoFin: integer;// Hora a partir de la cual NO uso  el auto
  Capacidad: NReal; // 12,16..20 kWh
  PMax_kW_carga, PMax_kW_descarga: NReal;
  //potencia máxima de demanda o inyección a la red
  renPerdidasPorHora: NREal; // ro
  nHorasTramo: integer // horas del tramo a simular EJ. 72 horas
  ): TPoblacion;

var
  aAuto: TdemandaConRespuesta;
  k, dia, hora, i: integer;
  vect_op, vect_dline: arreglos;
  Hora_inicio_uso, Hora_fin_uso, l: integer;
  E_ini_Auto, Econs_Auto, E_max_Auto, Emin_Auto, PMax_Gen_Auto,
  PMax_Dem_Auto, ren_Gen_Auto, ren_Dem_Auto, ro_Auto, C_Auto, F_Auto,
  L_Auto, Capacidad_Auto: NReal;
  ihora: integer;
  res: TPoblacion;
begin

  res:= TPoblacion.Create('Autos_Electricos', nHorasTramo );

  for k := 1 to Poblacion do
  begin
    Capacidad_Auto := Capacidad * (1 + 0.2 * (random - 0.5) / 0.5);
    //los parámetros corresponden a un e-min http://www.emin.com.uy/especificaciones.html
    E_ini_Auto := Capacidad_Auto * Random;  //(KWh)
    Econs_Auto := 1.5 + (1.5 + Random) * (Capacidad_Auto / 12);
    //(KWh)--> Asumo que recorre 40 Km diarios aprox.
    E_max_Auto := Capacidad_Auto;                   //(KWh) --->capacidad batería
    Emin_Auto := (ECons_Auto + 2 * (Capacidad_Auto / 12));
    //(KWh) ---> 3 kWh (40Km)+2 KWh de margen
    PMax_Gen_Auto := PMax_kW_descarga;   //(kW)  ----> para el e-min hay que poner 1.8 kW
    PMax_Dem_Auto := PMax_kW_carga; //(kWh)
    ren_Gen_Auto := 0.7;
    ren_Dem_Auto := 0.85;
    ro_Auto := renPerdidasPorHora;



    //inicializo vector_operabilidad=1

    for i := 1 to 168 do
      vect_op[i] := 1;

    //inicializo vector_deadline=0

    for i := 1 to 168 do
      vect_dline[i] := 0;

    //randomizo hora inicial entre HoraUsoInicio y HoraUsoFin
    Hora_inicio_uso := round(HoraUsoInicio - 1 + 2 * Random);

    Hora_fin_uso := round(HoraUsoFin - 1 + 2 * Random);

    //cargo los vectores de operabilidad y deadline segùn Hora_uso resultante
    for dia := 1 to 7 do
    begin
      for hora := 0 to 23 do
        if hora = Hora_inicio_uso then
        begin
          ihora := (hora + (dia - 1) * 24) mod 168 + 1;
          vect_dline[ihora - 1] := 1;
          for l := Hora_inicio_uso to Hora_fin_uso - 1 do
            vect_op[(l + (dia - 1) * 24) mod 168 + 1] := 0;
        end;
    end;

    aAuto := TdemandaConRespuesta.Create(E_ini_Auto, Econs_Auto,
      E_max_Auto, Emin_Auto, PMax_Gen_Auto, PMax_Dem_Auto, ren_Gen_Auto,
      ren_Dem_Auto, ro_Auto, vect_op, vect_dline);

    res.add(aAuto);
  end;
  Poblaciones.Add( res );
  result:= res;
end;

function CrearCalefones(Poblacion: integer; HoraUsoInicio, HoraUsoFin: integer;
  Volumen_lt: NReal; // 60lt, 90lt
  PMax_kW: NReal; renPerdidasPorHora: NREal; // ro
  nHorasTramo: integer // horas del tramo a simular EJ. 72 horas
  ): TPoblacion;

var
  aCalefon: TdemandaConRespuesta;
  k, dia, hora, i: integer;
  vect_op, vect_dline: arreglos;
  Hora_uso: integer;
  E_ini_calef, Econs_calef, E_max_calef, Emin_calef, PMax_Gen_calef,
  PMax_Dem_calef, ren_Gen_calef, ren_Dem_calef, ro_calef,
  C_Calef: NReal;
  ihora: integer;
  res: TPoblacion;

begin
  res:= TPoblacion.Create( 'Calefones', nHorasTramo );

  for k := 1 to Poblacion do
  begin
    C_calef := 28.7 {°C/kWh} * 30.0/Volumen_lt ;

    E_ini_calef := (10 + 20 * Random) / C_calef; //(ºC)
    Econs_calef := 40  / C_calef;             //(ºC)
    E_max_calef := 70  / C_calef;                   //(ºC)
    Emin_calef := 55  / C_calef;                     //(ºC)
    PMax_Gen_calef := 0;   //(kWh)
    PMax_Dem_calef := PMax_kW; //(kWh)
    ren_Gen_calef := 0.8;
    ren_Dem_calef := 0.9995;
    ro_calef := renPerdidasPorHora;
    // (unidades = ºC/KWh) (cuenta=1/(Ce*dens*V)) (28.7 es para un calefon de 30 lt)

    //inicializo vector_operabilidad=1

    for i := 1 to 168 do
      vect_op[i] := 1;

    //inicializo vector_deadline=0

    for i := 1 to 168 do
      vect_dline[i] := 0;

    //randomizo hora inicial entre HoraUsoInicio y HoraUsoFin
    Hora_uso := round(HoraUsoInicio + (HoraUsoFin - HoraUsoInicio) * Random);

    //cargo los vectores de operabilidad y deadline segùn Hora_uso resultante
    for dia := 1 to 7 do
    begin
      for hora := 0 to 23 do
        if hora = Hora_uso then
        begin
          ihora := (hora + (dia - 1) * 24) mod 168 + 1;
          vect_dline[ihora - 1] := 1;
          vect_op[ihora] := 0;
        end;
    end;

    aCalefon := TdemandaConRespuesta.Create(E_ini_calef, Econs_calef,
      E_max_calef, Emin_calef, PMax_Gen_calef, PMax_Dem_calef,
      ren_Gen_calef, ren_Dem_calef, ro_calef, vect_op, vect_dline);

    res.add(aCalefon);

  end;
  Poblaciones.Add( res );
  result:= res;
end;

constructor TdemandaConRespuesta.Create(E_ini, Econs, E_max, Emin,
  PMax_Gen, PMax_Dem, ren_Gen, ren_Dem, ro: NReal;
  vector_operabilidad, vector_deadline: arreglos);

begin
  self.E_ini := E_ini;
  self.Econs := Econs;
  self.E_max := E_max;
  self.Emin := Emin;
  self.PMax_Gen := PMax_Gen;
  self.PMax_Dem := PMax_Dem;
  self.ren_Gen := ren_Gen;
  self.ren_Dem := ren_Dem;
  self.ro := ro;
  self.vector_operabilidad := vector_operabilidad;
  self.vector_deadline := vector_deadline;
  self.E_finDelPaso := E_ini;
  peso_InicioHora := random;
  peso_FinHora := 1 - peso_InicioHora;
end;

procedure TdemandaConRespuesta.CDR_MW(
  var DCR_MW: TDAOFNReal; // Retorna la demanda en MW hora a hora.
  var Energias: TDAOfNReal; // Acumla la energía almacenada
  cmg_ve_ant, cmg_ve_: TDAofNReal;
  // INformación de costos marginales hora a hora.
  dtInicioDelTramo: TDateTime);
var
  texto: textfile;
  horas_pron: integer;
  fila, columna, j: integer;
  DiaDeLaSemanaInicioDelPaso, HoraDeInicioDelPaso: integer;
  indice_oper_deadline_inicial, indice_oper_deadline: integer;
  FijoEresEnSiguienteHora: boolean;
  i_lectura_simplex, a: integer;
  spx: TSimplex;

  cmg_ve: TDaOfNReal;
  lambda_E00, Lambda_E24, Lambda_E48: NReal;
  ValorEFinal, ValorEFinal_ant: NReal;
  cnt_IterValor_EFinal: integer;
  flg_Convergio: boolean;

begin

  ValorEFinal_ant := 0;
  ValorEFinal := 0;
  setlength(cmg_ve, length(cmg_ve_));
  for j := 0 to high(cmg_ve_) do
    cmg_ve[j] := cmg_ve_ant[j] * Peso_InicioHora + cmg_ve_[j] * Peso_FinHora;


  horas_pron := length(cmg_ve); //cantidad de horas del pronóstico
  DiaDeLaSemanaInicioDelPaso := DayOfWeek(dtInicioDelTramo);
  HoraDeInicioDelPaso := HourOfTheDay(dtInicioDelTramo);

  //indice que apunta a lo ingresado por el usuario en la ficha para cada dia y hora de la semana
  //la semana arranca el domingo=1
  indice_oper_deadline_inicial :=
    ((DiaDeLaSemanaInicioDelPaso - 1) * 24) + HoraDeInicioDelPaso + 1;

  //calcular dia de la semana y hora de inicio del paso a partir del datetime que nos va a pasar ruben
  //otra: no olvidarse de poner var P (arreglo de las potencias resultantes de las 72 horas) en el argumento de ResolverPaso

  cnt_IterValor_EFinal := 0;
  flg_Convergio := False;

  while not flg_Convergio and (cnt_IterValor_EFinal < 10) do
  begin

    if cnt_IterValor_EFinal = 0 then
      spx := TSimplex.Create_init(horas_pron + 1, horas_pron * 3 + 2, nil, nil)
    else
      spx.limpiar;

    for fila := 1 to horas_pron do
    begin
      //cargamos la primer submatriz (PDem)
      for columna := 1 to horas_pron do
      begin
        if (fila = columna) then
          spx.pon_e(fila, columna, - ren_Dem)
        else
          spx.pon_e(fila, columna, 0);
      end;

      //cargamos la segunda submatriz (PGem)
      for columna := horas_pron + 1 to 2 * horas_pron do
      begin
        if (fila = columna - horas_pron) then
          spx.pon_e(fila, columna, + 1/ ren_Gen )
        else
          spx.pon_e(fila, columna, 0);
      end;


      //cargamos la tercer submatriz  (E: energía acumulada en cada paso)
      for columna := 2 * horas_pron + 1 to 3 * horas_pron + 1 do
      begin
        if (fila = columna - 2 * horas_pron) then
          spx.pon_e(fila, columna, -ro)
        else if (fila = columna - 2 * horas_pron - 1) then
          spx.pon_e(fila, columna, 1)
        else
          spx.pon_e(fila, columna, 0);
      end;
    end;

    //cargamos termino independiente y fijamos restricción de igualdad
    indice_oper_deadline := indice_oper_deadline_inicial;

    FijoEresEnSiguienteHora := False;//si a una hora tengo deadline de carga, a la
    //siguiente asumo que se consumió (Econs) de esa carga

    for fila := 1 to horas_pron do
    begin
      if FijoEresEnSiguienteHora = True then
      begin
        spx.pon_e(fila, spx.nc, + Econs);
        FijoEresEnSiguienteHora := False;
      end
      else
      begin
        spx.pon_e(fila, spx.nc, 0 );
      end;

      if vector_deadline[indice_oper_deadline] = 1 then
        FijoEresEnSiguienteHora := True
      else
        FijoEresEnSiguienteHora := False;

      //el indice que apunta al vector de operabilidad tiene que ser circular
      if indice_oper_deadline + 1 = 169 then
        indice_oper_deadline := 1
      else
        indice_oper_deadline := 1 + indice_oper_deadline;
    end;

    //cargo fila objetivo de 1er submatriz PDEM
    for columna := 1 to horas_pron do
      spx.pon_e(spx.nf, columna, -cmg_ve[columna - 1]);

    //cargo fila objetivo de 2da submatriz PGEN
    for columna := horas_pron + 1 to 2 * horas_pron do
      spx.pon_e(spx.nf, columna, cmg_ve[columna - (horas_pron + 1)]);

    //cargo fila objetivo de 3er submatriz
    spx.pon_e(spx.nf, 3 * horas_pron + 1, ValorEFinal);


    indice_oper_deadline := indice_oper_deadline_inicial;

    spx.FijarVariable(2 * horas_pron + 1, E_ini);
    //fijamos la primer variable de E en la energía inicial (Eini)


    for fila := 1 to horas_pron do
      spx.FijarRestriccionIgualdad(fila);

    // Fijo las restricciones de energía mínima

    indice_oper_deadline := indice_oper_deadline_inicial;

    for j := 1 to horas_pron do
    begin
      if vector_operabilidad[indice_oper_deadline] = 0 then
      begin
        spx.FijarVariable(j, 0);
        spx.FijarVariable(j + horas_pron, 0);
      end
      else
      begin
        if PMax_Dem > 0.001 then
          spx.cota_sup_set(j, PMax_Dem)
        else
          spx.FijarVariable(j, 0);

        if PMax_Gen > 0.001 then
          spx.cota_sup_set(j + horas_pron, PMax_Gen)
        else
          spx.FijarVariable(j + horas_pron, 0);
      end;


      spx.cota_inf_set(j + 2 * horas_pron + 1,
        vector_deadline[indice_oper_deadline] * Emin);
      //seteo cotas inferiores de E en 0 o Emin segun haya consigna
      spx.cota_sup_set(j + 2 * horas_pron + 1, E_max);
      //seteo cotas superiores de E en Emax

      //el indice que apunta al vector de operabilidad tiene que ser circular
      if indice_oper_deadline + 1 = 169 then
        indice_oper_deadline := 1
      else
        indice_oper_deadline := 1 + indice_oper_deadline;

    end;


    //  spx.DumpSistemaToXLT_('prueba1.xlt', '');
    a := spx.resolver;
    if a <> 0 then
      raise Exception.Create('Solución infactible');

    lambda_E00 := spx.xmult(2 * horas_pron + 1);
//    lambda_E24 := spx.ymult( 1 );
    lambda_E24 := spx.ymult( 24);
    lambda_E48 := spx.ymult( 48);
    ValorEFinal := (lambda_E00 + Lambda_E24 + Lambda_E48) / 3.0;

    flg_COnvergio := (ValorEFinal - ValorEFinal_ant) < 0.001;

    if not flg_Convergio then
    begin
      inc( cnt_IterValor_EFinal );
      ValorEFinal_ant := ValorEfinal;
(*
      if ( cnt_IterValor_EFinal > 1 ) then
        writeln( 'cnt: ', cnt_IterValor_EFinal,', val: ', ValorEFinal:12:2 );
  *)
    end;

  end;


  //AHORA LEO LA SOLUCIÓN DEL SIMPLEX Y ESCRIBO EN : DCR_MW


  for i_lectura_simplex := 0 to horas_pron - 1 do
    DCR_MW[i_lectura_simplex] :=
      DCR_MW[i_lectura_simplex] + (spx.xval(i_lectura_simplex + 1) -
      spx.xval(i_lectura_simplex + horas_pron + 1)) / 1000.0;

  for i_lectura_simplex := 0 to horas_pron -1  do
    Energias[i_lectura_simplex] :=
      Energias[i_lectura_simplex] + spx.xval(i_lectura_simplex + 2* horas_pron +1 ) / 1000.0;

  //Actualizo el estado inicial Eini según el resultado del simplex
  E_finDelPaso := spx.xval(horas_pron * 2 + 2);

  // leo simplex

  spx.Free(True);
  setlength(cmg_ve, 0);
end;


initialization
  Poblaciones := TList.Create;
  dt_UltimaLLamada := -1;
  fuenteRuido := Tf_ddpUniformeRand3.Create(nil, 31);



end.
