unit ueditorPanelG;

{$mode delphi}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls,
  uauxiliares,
  xmatdefs, Math,
  usalasdejuego,
  uglobs,
  uDataSetGenerico,
  uvisordetabla,
  uactores,
  ugraficador,
  uregistradorVates;

type

  // Asocia un (Grafico) con un (TPaintBox)
  TRecGrafico = class
    panel: TPaintBox;
    grafico: TGrafico;
    constructor Create(aPanel: TPaintBox; aGrafico: TGrafico);
    procedure draw;
  end;

  // Lista de records como del tipo anterior.
  // por comodidad, la DEFAULT propertie devuelve directamente el Gráfico
  // para tener el registro k usar getRec(k)
  TRecGraficosLst = class(TList)
    procedure draw;
    procedure draw_panel(p: TPaintBox);
    procedure cambiar_eje_x(nuevoTitulo: string; x_min, x_max: NReal);
    function getRec(k: integer): TRecGrafico;
    constructor Create(aForm: TForm; NPaneles: integer; aSerieX: TSerie);
  private
    function Get(Index: integer): TGrafico;
    property Items[Index: integer]: TGrafico read Get; default;
  end;

  // Asocia, un par (Aactor->Variable) con un (RegistroVates).
  TRecSerie = class
    actor, variable: string;
    Serie: TSerie;
    registroVates: TRegistroVates;
    recGrafico: TRecGrafico;
    // Si data <> nil crea un registroVates para armar la serie, sino simplemente usa data
    constructor Create(aActor, aVariable, etiqueta: string; aRecGrafico: TRecGrafico; data: TDAOfNReal );
  end;

  // Lista de Records como en lanterior
  TRecSeriesLst = class(TList)

  private
    panelesG: TRecGraficosLst;
    function Get(Index: integer): TRecSerie;
    property Items[Index: integer]: TRecSerie read Get; default;

  public
    constructor Create( panelesG: TRecGraficosLst );
    // retorna la sublista de registros asociados a un panel dado
    function getInPanelLst( pbx: TPaintBox ): TRecSeriesLst;

    // Si data<> nil entonces se trata de un vector ya creado y arma la serie con eso
    // y no crea un registroVates. En esta Caso Actor y Variable pueden ser cualquier cosa y
    // se usan solo para poner nombre la a serie como "Actor.Variable".
    function AddRec(kPanelG: integer; Actor, Variable, Etiqueta: string; data: TDAOfNReal = nil;
       kEje: integer = 0 ): TRecSerie;
  end;


  TListadoSeries = class ( TTabla )
    public
      flg_populate: boolean;

      recSeries: TRecSeriesLst;
      panel: TPaintBox;

      constructor Create(
                  AOwner: TComponent; Name: string;
                  aRecSeries: TRecSeriesLst; aPanel: TPaintBox ); reintroduce;

      procedure Editar_onClick(nidRec: string; kFila: integer); override;
      procedure SemaforoRojo_OnClick(nidRec: string; kFila: integer); override;
      procedure SemaforoVerde_OnClick(nidRec: string; kFila: integer); override;
      procedure Lupa_OnClick(nidRec: string; kFila: integer); override;

      class function CrearNuevoProblema: integer;virtual;abstract;

      procedure Borrar_OnClick( nidRec: string; kFila: integer ); override;
      procedure Populate;
      procedure Paint; override;
  end;



  { TfEditarPanelGrafico }

  TfEditarPanelGrafico = class(TForm)
    btAgregar: TButton;
    cbActores: TComboBox;
    cbCanal: TComboBox;
    cbVars: TComboBox;
    gbActores: TGroupBox;
    ScrollBox1: TScrollBox;
    Series: TGroupBox;
    procedure cbActoresChange(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    sala: TSalaDeJuego;
    PanelesG: TRecGraficosLst;
    recSeriesLst: TRecSeriesLst;
    listadoSeries: TListadoSeries;
    PaintBox: TPaintBox;
    procedure InicializarForm(
                sala: TSalaDeJuego;
                RecSeriesLst: TRecSeriesLst;
                aPaintBox: TPaintBox );
  end;



var
  fEditarPanelGrafico: TfEditarPanelGrafico;


implementation

{$R *.lfm}



constructor TRecGrafico.Create(aPanel: TPaintBox; aGrafico: TGrafico);
begin
  inherited Create;
  self.panel := aPanel;
  self.grafico := aGrafico;
end;

procedure TRecGrafico.draw;
begin
  grafico.Draw(Panel.Canvas, Panel.Width, Panel.Height);
end;


procedure TRecGraficosLst.draw;
var
  k: integer;
begin
  for k := 0 to Count - 1 do
    TRecGrafico(inherited items[k]).draw;
end;


procedure TRecGraficosLst.cambiar_eje_x(nuevoTitulo: string; x_min, x_max: NReal);
var
  k: integer;
begin
  for k := 0 to Count - 1 do
  begin
    items[k].eje_x.titulo := nuevoTitulo;
    items[k].eje_x.nDivs := 72 div 6;
    items[k].eje_x.Forzar_vmax(x_max);
    items[k].eje_x.Forzar_vmin(x_min);
  end;
end;



function TRecGraficosLst.getRec(k: integer): TRecGrafico;
begin
  Result := inherited items[k];
end;

constructor TRecGraficosLst.Create(aForm: TForm; NPaneles: integer; aSerieX: TSerie);
var
  k: integer;
  aGrafico: TGrafico;
  aPanel: TPaintBox;

begin
  inherited Create;
  (*** Creo los gráficos y les asocio paneles. ***)
  for k := 0 to NPaneles - 1 do
  begin
    aGrafico := TGrafico.Create('Panel ' + IntToStr(k), TG_DispersionXY, 1);
    aGrafico.FontSize := 8;
    aGrafico.RadioMarcador := 2;
    aGrafico.AddSerie(aSerieX);
    aPanel := aForm.FindComponent('pbx' + IntToStr(k)) as TPaintBox;
    Add(TRecGrafico.Create(aPanel, aGrafico));
  end;
end;



procedure TRecGraficosLst.draw_panel(p: TPaintBox);
var
  k: integer;
  aRec: TRecGrafico;
begin
  for k := 0 to Count - 1 do
  begin
    aRec := inherited items[k];
    if aRec.panel = p then
      aRec.draw;
  end;
end;



function TRecGraficosLst.Get(Index: integer): TGrafico;
begin
  Result := TRecGrafico(inherited Items[Index]).grafico;
end;



constructor TRecSerie.Create(aActor, aVariable, Etiqueta: string; aRecGrafico: TRecGrafico; data: TDAOfNReal);
begin
  self.Actor := aActor;
  self.Variable := aVariable;
  self.recGrafico := aRecGrafico;
  if data = nil then
  begin
    registroVates := uregistradorVates.CrearRegistro(Actor, Variable);
    if registroVates = nil then
       raise Exception.Create('No encontró variable: '+Actor+'.'+Variable );
    Serie := TSerie.Create( Etiqueta, registroVates.serie_VE,
          nil, RecGrafico.grafico);
  end
  else
    Serie := TSerie.Create( Etiqueta, data,
        nil, RecGrafico.grafico);
  Serie.tipoGrafico := TG_DispersionXY;
  recGrafico.grafico.addSerie(Serie);
end;


function TRecSeriesLst.Get(Index: integer): TRecSerie;
begin
  Result := TRecSerie(inherited Items[index]);
end;


constructor TRecSeriesLst.Create( panelesG: TRecGraficosLst );
begin
  inherited Create;
  self.panelesG:= panelesG;
end;

// retorna la sublista de registros asociados a un panel dado
function TRecSeriesLst.getInPanelLst( pbx: TPaintBox ): TRecSeriesLst;
var
  res: TRecSeriesLst;
  aRecSerie: TRecSerie;
  k: integer;
begin
  res:= TRecSeriesLst.Create( panelesG );
  for k:= 0 to count-1 do
  begin
    aRecSerie:= get( k );
    if aRecSerie.recGrafico.panel = pbx then
      res.add( aRecSerie );
  end;
  result:= res;
end;


function TRecSeriesLst.AddRec(kPanelG: integer; Actor, Variable, Etiqueta: string;
            data: TDAOfNReal=nil;
            kEje: integer=0 ): TRecSerie;
var
  res: TRecSerie;
  recGrafico: TRecGrafico;

begin
  recGrafico:= panelesG.getRec( kPanelG );
  res:= TRecSerie.Create( Actor, Variable, Etiqueta, recGrafico, data );
  res.Serie.eje:= kEje;
  add( res );
  result:= res;
end;

procedure TfEditarPanelGrafico.InicializarForm(
            sala: TSalaDeJuego;
            RecSeriesLst: TRecSeriesLst;
            aPaintBox: TPaintBox );
begin
  self.Sala := Sala;
  self.PanelesG:= PanelesG;
  self.RecSeriesLst:= RecSeriesLst;
  cbActores.Items := sala.listaActores.nombresCosas;

  listadoSeries:= TListadoSeries.Create( self, 'listadoSeries', RecSeriesLst, PaintBox );
  listadoSeries.Parent:= scrollbox1;
  listadoSeries.flg_populate:= false;
end;



procedure TfEditarPanelGrafico.cbActoresChange(Sender: TObject);
var
  aActor: TActor;
  nombreActor: string;
begin
  nombreActor := cbActores.Text;
  aActor := Sala.GetActorPorNombre(nombreActor);
  if aActor <> nil then
  begin
    cbVars.Items := aActor.getNombresPublicados;
  end;
end;




constructor TListadoSeries.Create(
  AOwner: TComponent; Name: string; aRecSeries: TRecSeriesLst; aPanel: TPaintBox );
begin
  inherited Create(AOwner, Name, 4, 1, 0 );
  flg_populate:= true;
  recSeries:= aRecSeries;
  panel:= aPanel;
end;



procedure TListadoSeries.Editar_onClick(nidRec: string; kFila: integer);
begin

end;

procedure TListadoSeries.SemaforoRojo_OnClick(nidRec: string; kFila: integer);
begin

end;

procedure TListadoSeries.SemaforoVerde_OnClick(nidRec: string; kFila: integer);
begin

end;

procedure TListadoSeries.Lupa_OnClick(nidRec: string; kFila: integer);
begin

end;

class function CrearNuevoProblema: integer;
begin

end;

procedure TListadoSeries.Paint;
begin
  if flg_populate then Populate;
  inherited Paint;
end;


procedure TListadoSeries.Populate;
var
  k: integer;
  snid: string;
  estado: integer;
  r: TRecSerie;
  lst: TRecSeriesLst;
begin
  lst:= self.recSeries.getInPanelLst( self.panel );

  ClearRedim( lst.count + 1, 7);
  // fila encabezado
  wrTexto('', 0, 0, 'Actor');
  wrTexto('', 0, 1, 'Variable');
  wrTexto('', 0, 2, 'tipo');
  wrTexto('', 0, 3, 'color');

  AlinearCelda(0, 0, CAH_Izquierda, CAV_Centro);
  AlinearCelda(0, 1, CAH_Izquierda, CAV_Centro);
  AlinearCelda(0, 2, CAH_Izquierda, CAV_Centro);
  AlinearCelda(0, 3, CAH_Izquierda, CAV_Centro);


  Self.SetBgColorFila(0, clNavy);
  Self.SetFontColorFila(0, clYellow);

  Self.SetBgColorCelda(0, 0, clNavy);
  Self.SetBgColorCelda(0, 1, clNavy);

  for k:= 0 to lst.count -1 do
  begin
    r:= lst[k];
    wrTexto('', k, 0, r.actor);
    wrTexto('', k, 1, r.variable);
    wrTexto('', k, 2,  EjeToStr( r.Serie.eje ));
    wrTexto('', k, 3, ColorToString( r.Serie.colorLinea ));
    AlinearCelda(k, 0, CAH_Izquierda, CAV_Centro);
    AlinearCelda(k, 1, CAH_Izquierda, CAV_Centro);
    AlinearCelda(k, 2, CAH_Izquierda, CAV_Centro);
    AlinearCelda(k, 3, CAH_Izquierda, CAV_Centro);

    // ahora la botonera en la última columna

    wrBotonera(snid, k, 4,
      [iid_Editar, iid_borrar, iid_subir, iid_bajar],
      [hintPorDefecto(iid_Editar),
      hintPorDefecto(iid_Borrar),
      hintPorDefecto(iid_subir),
      hintPorDefecto(iid_bajar)]);

    if (estado <= 0) then
      wrBotonera(snid, k, 5, [iid_SemaforoRojo, iid_Lupa], ['Activar', 'Consultar'])
    else
      wrBotonera(snid, k, 5, [iid_SemaforoVerde, iid_Lupa], ['Desactivar', 'Consultar']);

    if (k mod 2) = 0 then
      if estado <= 0 then
        SetBgColorFila(k, clFondoFilaPar_Inactiva)
      else
        SetBgColorFila(k, clFondoFilaPar_Activa)
    else
    if estado <= 0 then
      SetBgColorFila(k, clFondoFilaImpar_Inactiva)
    else
      SetBgColorFila(k, clFondoFilaImpar_Activa);
  end;
  reposicionar;
end;

procedure TListadoSeries.Borrar_OnClick( nidRec: string; kFila: integer );
begin
  //??? ver

  flg_populate:= true;
  Invalidate;
  //  Populate; // prefiero polular
end;




end.


