unit uregistradorvates;

{$mode delphi}

interface

uses
  Classes, SysUtils,
  uauxiliares,
  ucosa, ucosaconnombre,
  uglobs, usalasdejuego, xMatDefs, uVarDefs, ugraficador;

type

  TRegistroVates = class
    varDef: TVarDef;
    canalGrafico: integer;
    PEa, PEb: NReal;
    NCronicas, NPuntosT: integer;
    ValoresPorPaso: TDAOfDAOfNReal;
    serie_PEa, serie_PEb, serie_VE: TDAOfNReal;
    constructor Create(varDef: TVarDef; PEa, PEb: NReal;
    // Probabilidades de excedencia a graficar
      canalGrafico: integer; NCronicas, nPuntosT: integer);
    procedure Muestrear(kPaso, kCronica: integer);
    function GetVal(kPaso, kCronica: integer): NReal;
    procedure Resumir; // Calcula serie_PEa, serie_PEb y serie_VE
    procedure Free;
  end;

var
  RegistrosVates: TList;

// Fija la Sala de Juego sobre la que se realizan los muestreos.
procedure MuestrarAlFinDeCadaPasoSimOnSala(sala: TSalaDeJuego);

// AlFinal de la simulación. Hace el resumen de todos los registros Vates.
procedure Resumir;

function CrearRegistro(Actor, Variable: string): TRegistroVates;

implementation

var
  LaSala: TSalaDeJuego;


procedure Muestrear;
var
  a: TRegistroVates;
  k: integer;
  kPaso, kCronica: integer;
begin
  kPaso := LaSala.globs.kPaso_ - 1;
  kCronica := LaSala.globs.kCronica - 1;
  for k := 0 to RegistrosVates.Count - 1 do
  begin
    a := RegistrosVates[k];
    a.Muestrear(kPaso, kCronica);
  end;
end;


// AlFinal de la simulación. Hace el resumen de todos los registros Vates.
procedure Resumir;
var
  a: TRegistroVates;
  k: integer;
begin
  for k := 0 to RegistrosVates.Count - 1 do
  begin
    a := RegistrosVates[k];
    a.resumir;
  end;
end;

function CrearRegistro(Actor, Variable: string): TRegistroVates;
var
  aReg: TRegistroVates;
  aVarDef: TVarDef;
  aCosa: TCosaConNombre;
  lstres: TListaDeCosasConNombre;

begin
  Result := nil;
  if Actor = '-' then
    aCosa:= LaSala
  else
   aCosa := LaSala.BuscarPorNombre(Actor, lstres);
  if aCosa = nil then
    exit;
  aVarDef := aCosa.buscarVariable(Variable);
  if aVarDef = nil then
    exit;

  aVarDef.Prepararse;
  aReg := TRegistroVates.Create(aVarDef, 0.1, 0.9, 0,
    LaSala.globs.NCronicasSim, LaSala.globs.NPasos);

  RegistrosVates.Add(aReg);
  Result := aReg;
end;


procedure MuestrarAlFinDeCadaPasoSimOnSala(sala: TSalaDeJuego);
begin
  LaSala := sala;
  sala.globs.procNot_FinPaso := Muestrear;
end;

constructor TRegistroVates.Create(varDef: TVarDef; PEa, PEb: NReal;
  // Probabilidades de excedencia a graficar
  canalGrafico: integer; NCronicas, NPuntosT: integer);
var
  k: integer;
begin
  inherited Create;
  self.PEa:= PEa;
  self.PEb:= PEb;
  self.varDef := varDef;
  self.CanalGrafico := CanalGrafico;
  self.NPuntosT := NPuntosT;
  self.NCronicas:= NCronicas;
  setlength(ValoresPorPaso, NPuntosT);
  for k := 0 to NPuntosT - 1 do
    setlength(ValoresPorPaso[k], NCronicas);
  setlength(serie_PEa, NPuntosT);
  setlength(serie_PEb, NPuntosT);
  setlength(serie_VE, nPuntosT);
end;

procedure TRegistroVates.Muestrear(kPaso, kCronica: integer);
begin
// writeln( 'kPaso: ', kPaso, ', kCronica: ', kCronica );
  ValoresPorPaso[kPaso][kCronica] := varDef.asNR;
end;

function TRegistroVates.GetVal(kPaso, kCronica: integer): NReal;
begin
 result:= ValoresPorPaso[kPaso][kCronica];
end;


procedure TRegistroVates.Resumir;
var
  k: integer;
  a: TDAOfNReal;
begin
  setlength(a, NCronicas);
  for k := 0 to NPuntosT - 1 do
  begin
    a := copy(ValoresPorPaso[k]);
    QuickSort_Decreciente(a);
    serie_VE[k] := vprom(a);
    serie_PEa[k] := a[trunc(PEa * (NCronicas - 0.5))];
    serie_PEb[k] := a[trunc(PEb * (NCronicas - 0.5))];
  end;
  setlength(a, 0);
end;

procedure TRegistroVates.Free;
var
  k: integer;
begin
  setlength(serie_VE, 0);
  setlength(serie_PEb, 0);
  setlength(serie_PEa, 0);
  for k := 0 to high(NPuntosT) do
    setlength(ValoresPorPaso[k], 0);
  setlength(ValoresPorPaso, 0);
end;

initialization
  RegistrosVates := TList.Create;

end.


