unit uparqueeolicodata;

{$mode delphi}

interface

uses
  Classes,
  SysUtils,
  usqlutils,
  xmatdefs,
  Math,
  umeteoblue,
  upgsql,
  udbconpg;

type
  TParqueEolico = class
    nid: integer;  // identificador en tabla parques eolicos

    top, left: integer;
    Activo: integer; // -1 Parece que no se hace, 0 En construcción, 1 En Operación.

    CentralGeneradora: string;
    AgenteGenerador: string;
    EstadoActual: string;
    Departamento: string;
    Localidad: string;
    FechaEntradaEnServicio: TDateTime;
    Tension_kV: string;
    Potencia_Autorizada_MW: double;
    Latitud: double;
    Longitud: double;
    Cota_asl: double; // cota sobre el nivel del mar para consultas mb.
    Fabricante: string;
    Modelo: string;
    PotenciaAerogenerador_MW: double;
    CantidadAerogeneradores: integer;

    // PRONOSTICO_GEN
    nidProno: integer; // pronóstico usado
    dt_PrimerHora: TDateTime;
    P: TDAOfNReal;

    constructor Create(Activo: integer; CentralGeneradora: string;
      AgenteGenerador: string; EstadoActual: string; Departamento: string;
      Localidad: string; FechaEntradaEnServicio: string; // En formato de Fecha Local
      Tension_kV: string; Potencia_Autorizada_MW: double; Latitud: double;
      Longitud: double; Cota_asl: double; Fabricante: string; Modelo: string;
      PotenciaAerogenerador_MW: double; CantidadAerogeneradores: integer);

    constructor CreateFromDBRec( ds: TSQLQuery );


    function GetSQLTableHelper: TSQLTableHelper;
    class procedure WriteEncabToXLT(var f: textfile);
    procedure WriteToXLT(var f: textfile);

    // retorna el nid del Pronóstico usado. (-1 indica no calculó)
    function CalcPronosticoGen(db: TDBPqCon): integer;

    // Calcula en forma MUY SENCILLA una estimación de la potencia del parque
    function VelToPot_SIMPLE_(v_mps: double): double;

  end;


implementation

constructor TParqueEolico.Create(Activo: integer; CentralGeneradora: string;
  AgenteGenerador: string; EstadoActual: string; Departamento: string;
  Localidad: string; FechaEntradaEnServicio: string; Tension_kV: string;
  Potencia_Autorizada_MW: double; Latitud: double; Longitud: double;
  Cota_asl: double;
  Fabricante: string; Modelo: string; PotenciaAerogenerador_MW: double;
  CantidadAerogeneradores: integer);
begin
  self.nid:= -1; // indica no identificado en la DB. Puede ser que no exista o
   // que exista la CentralGeneradora en la DB.
  Self.Activo := Activo;
  Self.CentralGeneradora := CentralGeneradora;
  Self.AgenteGenerador := AgenteGenerador;
  Self.EstadoActual := EstadoActual;
  Self.Departamento := Departamento;
  Self.Localidad := Localidad;
  if (FechaEntradaEnServicio <> '') then
    Self.FechaEntradaEnServicio := StrToDateTime(FechaEntradaEnServicio)
  else
    Self.FechaEntradaEnServicio := StrToDateTime('1/1/3000');

  Self.Tension_kV := Tension_kV;
  Self.Potencia_Autorizada_MW := Potencia_Autorizada_MW;
  Self.Latitud := Latitud;
  Self.Longitud := Longitud;
  self.Cota_asl:= Cota_asl;
  Self.Fabricante := Fabricante;
  Self.Modelo := Modelo;
  Self.PotenciaAerogenerador_MW := PotenciaAerogenerador_MW;
  Self.CantidadAerogeneradores := CantidadAerogeneradores;
end;


constructor TParqueEolico.CreateFromDBRec(ds: TSQLQuery);
begin
  nid:= ds.FieldByName('nid' ).AsInteger;
  activo:= ds.FieldByName( 'Activo' ).AsInteger;
  CentralGeneradora:= ds.FieldByName( 'CentralGeneradora' ).AsString;
  AgenteGenerador:= ds.FieldByName( 'AgenteGenerador' ).AsString;
  EstadoActual:= ds.FieldByName( 'EstadoActual' ).AsString;
  Departamento:= ds.FieldByName( 'Departamento' ).AsString;
  Localidad:= ds.FieldByName( 'Localidad' ).AsString;
  FechaEntradaEnServicio:= ds.FieldByName( 'FechaEntradaEnServicio' ).AsDateTime;
  Tension_kV:= ds.FieldByName( 'Tension_kV' ).AsString;
  Potencia_Autorizada_MW:= ds.FieldByName( 'Potencia_Autorizada_MW' ).AsFloat;
  Latitud:= ds.FieldByName( 'Latitud' ).AsFloat;
  Longitud:= ds.FieldByName( 'Longitud' ).AsFloat;
  Cota_asl:= ds.FieldByName( 'Cota_asl' ).AsFloat;

  Fabricante:= ds.FieldByName( 'Fabricante' ).AsString;
  Modelo:= ds.FieldByName( 'Modelo' ).AsString;
  PotenciaAerogenerador_MW:= ds.FieldByName( 'PotenciaAerogenerador_MW' ).AsFloat;
  CantidadAerogeneradores:= ds.FieldByName( 'CantidadAerogeneradores' ).AsInteger;
end;

function TParqueEolico.GetSQLTableHelper: TSQLTableHelper;
var
  ib: TSQLTableHelper;
begin
  inherited Create;
  ib := TSQLTableHelper.Create('parqueseolicos', 'nid');
  IB.add('nid', nid);
  IB.add('Activo', activo);
  IB.add('CentralGeneradora', CentralGeneradora);
  IB.add('AgenteGenerador', AgenteGenerador);
  IB.add('EstadoActual', EstadoActual);
  IB.add('Departamento', Departamento);
  IB.add('Localidad', Localidad);
  IB.add('FechaEntradaEnServicio', FechaEntradaEnServicio);
  IB.add('Tension_kV', Tension_kV);
  IB.add('Potencia_Autorizada_MW', Potencia_Autorizada_MW);
  IB.add('Latitud', Latitud);
  IB.add('Longitud', Longitud);
  IB.add('Fabricante', Fabricante);
  IB.add('Modelo', Modelo);
  IB.add('PotenciaAerogenerador_MW', PotenciaAerogenerador_MW);
  IB.add('CantidadAerogeneradores', CantidadAerogeneradores);
  Result := ib;
end;

class procedure TParqueEolico.WriteEncabToXLT(var f: textfile);
begin
  Write(f, 'nid');
  Write(f, #9, 'Activo');
  Write(f, #9, 'CentralGeneradora');
  Write(f, #9, 'AgenteGenerador');
  Write(f, #9, 'EstadoActual');
  Write(f, #9, 'Departamento');
  Write(f, #9, 'Localidad');
  Write(f, #9, 'FechaEntradaEnServicio');
  Write(f, #9, 'Tension_kV');
  Write(f, #9, 'Potencia_Autorizada_MW');
  Write(f, #9, 'Latitud');
  Write(f, #9, 'Longitud');
  Write(f, #9, 'Fabricante');
  Write(f, #9, 'Modelo');
  Write(f, #9, 'PotenciaAerogenerador_MW');
  Write(f, #9, 'CantidadAerogeneradores');
  writeln(f);
end;


procedure TParqueEolico.WriteToXLT(var f: textfile);
begin
  Write(f, nid);
  Write(f, #9, Activo);
  Write(f, #9, CentralGeneradora);
  Write(f, #9, AgenteGenerador);
  Write(f, #9, EstadoActual);
  Write(f, #9, Departamento);
  Write(f, #9, Localidad);
  Write(f, #9, FechaEntradaEnServicio);
  Write(f, #9, Tension_kV);
  Write(f, #9, Potencia_Autorizada_MW);
  Write(f, #9, Latitud);
  Write(f, #9, Longitud);
  Write(f, #9, Fabricante);
  Write(f, #9, Modelo);
  Write(f, #9, PotenciaAerogenerador_MW);
  Write(f, #9, CantidadAerogeneradores);
  writeln(f);
end;

function TParqueEolico.VelToPot_SIMPLE_(v_mps: double): double;
var
  k: integer;
  pu: NReal;
begin
  if v_mps <= 3.3 then
    pu := 0
  else if v_mps >= 15 then
    pu := 1;
  pu := (1.0 / (1.0 + EXP(-0.594709117 * (v_mps - 7))));
  Result := pu * Potencia_Autorizada_MW;
  //Result :=  min( pu * PotenciaAerogenerador_MW * CantidadAerogeneradores * 0.97, Potencia_Autorizada_MW );
end;



function TParqueEolico.CalcPronosticoGen( db: TDBPqCon ): integer;
var
  aESR: TEnergySimpleRec;
  archi: string;
  nHoras: integer;
  k: integer;
  aRec: TEnergySimpleDataRec;
begin
  nidProno:= db.funcI( 'SELECT nid FROM mb_energysimple  ORDER BY dtPrimeraMuestra DESC LIMIT 1 ');
  if nidProno < 0 then
  begin
    setlength( P, 0 );
    result:= nidProno;
    exit;
  end;
  aESR := TEnergySimpleRec.CreateFromDB( db, nidProno);
  dt_PrimerHora := aESR.dtPrimerMuestra;
  setlength(P, aESR.NHoras);
  for k := 0 to high(P) do
  begin
    aRec := aESR.rec(k);
    P[k] := VelToPot_SIMPLE_(aRec.v_mps);
  end;
  aESR.Free;
  result:= nidProno;
end;



end.


