unit uparques_modelocombinado;

{$mode delphi}

interface

uses
  Classes, SysUtils,
  Math, xmatdefs, matreal, ucalibrador,
  uparques_modeloempirico, uauxiliares,
  umodelosintcegh, ucaminos_modelado_ro,
  ucomplex, uro_aire, ufechas;

type

  { TRecFechasRestriccion }

  TRecFechasRestriccion = record
    FechaDesde, FechaHasta: TDateTime;

  end;
  TRecEnergiaPorFecha = record
    fecha: TDateTime;
    energia:NReal;
  end;
  TDAofRecsFechas = array of TRecFechasRestriccion;
  TDAofRecsEnergiaPorFecha = array of TRecEnergiaPorFecha;

  procedure AddItem (Fecha:TDateTime;energia:NReal;var TRecsEnergiaPorFecha:TDAofRecsEnergiaPorFecha);
  procedure EnergiaPorFecha_To_XLT (nombreArchi: String;TRecsEnergiaPorFecha:
    TDAofRecsEnergiaPorFecha;factor_smec_scada:NReal);
  function Energia10mToEnergiaHoraria (RecsEnergia10m:TDAofRecsEnergiaPorFecha):TDAofRecsEnergiaPorFecha;
  procedure SumarEnergiaAultimoItem (e:NReal; var TRecsEnergiaPorFecha:TDAofRecsEnergiaPorFecha);

  procedure EnergiaPorFecha_To_DTE(nombreArchi: String;
    RecsEnergiaPorFecha: TDAofRecsEnergiaPorFecha; factor_smec_scada: NReal
    ;fechaDesde,fechaHasta:TDateTime);


type

  { TParque_ModeloCombinado }

  TParque_ModeloCombinado = class(TCalibradorParque)


    // Matriz con todos los valores ME/MT. Se mantienen los de la MatrizBorde
    //y el resto se aproxima por método iterativo.
    MatrizFactor: TMatR;

    constructor CreateLoadFromArchiCalibrador(archiCalibrador: string);
    constructor CreateLoadFromArchi(archi: string);
    procedure StoreInArchi(archi: string);

    // Procedimientos para calcular factores a multiplicar al MT según cluster.
    procedure Calc_MatrizFactores_ME_MT;
    procedure CalcularMatrizBorde(var MatrizBorde: TMatR);
    procedure Barrido_ImponerBorde(var MatrizFactor: TMatR; MatrizBorde: TMatR);
    function Barrido_PromediarCasilleros(var MatrizFactor: TMatR;
      MatrizBorde: TMatR): NReal;
    procedure Calcular_PE(archi_series: string; dtDesde, dtHasta, dtDesdeCalcRed,
      dtHastaCalcRed: TDateTime; factor_smec_scada: NReal;var RecsEnergia10m:TDAofRecsEnergiaPorFecha );

    // Calcula la potencia eléctrica con y sin restriciones operativas
    function Calc_PE_ConYSinRO(var PE_Con_RO, PE_Sin_RO: NReal;
      v_anemo, dir_Dato, ro_aire_Dato: NReal; cgm_Dato: NReal): boolean;


  end;

implementation

procedure AddItem(Fecha: TDateTime; energia: NReal;
  var TRecsEnergiaPorFecha: TDAofRecsEnergiaPorFecha);
var
  NDatos:integer;
begin
  SetLength(TRecsEnergiaPorFecha, Length(TRecsEnergiaPorFecha)+1);
  NDatos:=Length(TRecsEnergiaPorFecha)-1;
  TRecsEnergiaPorFecha[NDatos].fecha:=fecha;
  TRecsEnergiaPorFecha[NDatos].energia:=energia;
end;

procedure EnergiaPorFecha_To_XLT(nombreArchi: String;
  TRecsEnergiaPorFecha: TDAofRecsEnergiaPorFecha; factor_smec_scada: NReal);
var f:TextFile;
    i:integer;
    E_Total_scada:NReal;
begin
  assignfile(f, nombreArchi);
  rewrite(f);

  writeln(f,'Fecha', #9, ' Energía no suministrada SCADA [MWh]',#9, ' Factor SMEC/SCADA [p.u] '
    , #9, ' Energía no suministrada SMEC [MWh]');

  E_Total_scada:=0;

  for i := 0 to high(TRecsEnergiaPorFecha) do
  begin
    E_Total_scada:=E_Total_scada+TRecsEnergiaPorFecha[i].energia;
    WriteLn(f,TRecsEnergiaPorFecha[i].fecha,#9,TRecsEnergiaPorFecha[i].energia,#9,
      factor_smec_scada,#9,TRecsEnergiaPorFecha[i].energia*factor_smec_scada);
  end;

  writeln(f,'E[MWh] totales', #9, E_Total_scada,#9, ''
    , #9, E_Total_scada*factor_smec_scada);


  CloseFile(f);

end;

function Energia10mToEnergiaHoraria(
  RecsEnergia10m: TDAofRecsEnergiaPorFecha): TDAofRecsEnergiaPorFecha;
var
 N10min,i:integer;
 res:TDAofRecsEnergiaPorFecha;
 dt_hora_actual,dt:TDateTime;
begin
  N10min:=Length(RecsEnergia10m);

  dt_hora_actual:=trunc(RecsEnergia10m[0].fecha)+trunc(frac(RecsEnergia10m[0].fecha)*24)/24;
  AddItem(dt_hora_actual,RecsEnergia10m[0].energia,res);

  for i:=1 to N10min-1 do
  begin
    dt:=RecsEnergia10m[i].fecha;
    if (dt>dt_hora_actual+dt_un_Minuto*5) and (dt<(dt_hora_actual+1/24+dt_un_Minuto*5)) then
      SumarEnergiaAultimoItem(RecsEnergia10m[i].energia,res)
    else
    begin
      dt_hora_actual:=trunc(RecsEnergia10m[i].fecha)+trunc(frac(RecsEnergia10m[i].fecha)*24)/24;
      AddItem(dt_hora_actual,RecsEnergia10m[i].energia,res);
    end;
  end;

  Result:=res;
end;

procedure SumarEnergiaAultimoItem(e: NReal;
  var TRecsEnergiaPorFecha: TDAofRecsEnergiaPorFecha);
var
 NultimoItem:integer;
 Eanterior:NReal;
begin
  NultimoItem:=Length(TRecsEnergiaPorFecha)-1;
  Eanterior:=TRecsEnergiaPorFecha[NultimoItem].energia;
  TRecsEnergiaPorFecha[NultimoItem].energia:=Eanterior+e;
end;

procedure EnergiaPorFecha_To_DTE(nombreArchi: String;
  RecsEnergiaPorFecha: TDAofRecsEnergiaPorFecha; factor_smec_scada: NReal
  ;fechaDesde,fechaHasta:TDateTime);
var
  idx_recs,i,Nfechas,j,fila,columna: Integer;
  f:TextFile;
  dt_fecha_actual:TDateTime;
  Mat_Res:TMatR;
  acum_diario,dist:NReal;

begin

  Nfechas:=trunc(fechaHasta-fechaDesde)+1;
  Mat_Res:=TMatR.Create_Init(Nfechas,24);

  Mat_Res.Ceros;

  for i:=0 to High(RecsEnergiaPorFecha) do
  begin
    dist:=(RecsEnergiaPorFecha[i].fecha-fechaDesde)*24;
    fila:=trunc(dist/24)+1;
    columna:=round(frac(dist/24)*24)+1;
    Mat_Res.pon_e(fila,columna,RecsEnergiaPorFecha[i].energia*factor_smec_scada);
  end;


  assignfile(f, nombreArchi);
  rewrite(f);

  write(f,'Fecha',#9,'Total');

  for i:=1 to 24 do
     Write(f,#9,i);

  WriteLn(f);


  for i:=1 to Nfechas do
  begin
    dt_fecha_actual:=fechaDesde+i-1;
    Write(f,FormatDateTime('dd-mm-yyyy',dt_fecha_actual));
    acum_diario:=Mat_Res.Fila(i).SumaDesdeHasta(1,24);
    Write(f,#9,acum_diario);

    for j:=1 to 24 do
    begin
      Write(f,#9,Mat_Res.e(i,j));

      WriteLn(IntToStr(i) + ' - ' + IntToStr(j) + ' = ' + FloatToStr(Mat_Res.e(i,j)));


    end;

    WriteLn(f);

  end;

  CloseFile(f);

end;

  //ESC 2 FOR ANIDADOS PARA PARA RECORRER TODO

  //write(f,'Fecha');
  //
  //for i:=1 to 24 do
  //  Write(f,#9,i);
  //
  //WriteLn(f);
  //
  //Nfechas:=trunc(fechaHasta-fechaDesde)+1;
  //
  //idx_recs:=0;
  //fecha_actual:=trunc(fechaDesde);
  //
  //for i := 1 to Nfechas do
  //begin
  //  write(f,fecha_actual);
  //  for j:=0 to 23 do
  //  begin
  //    fecha_actual:=fecha_actual+j/24;
  //    if trunc(RecsEnergiaPorFecha[idx_recs].fecha-fecha_actual)=0 then
  //    begin
  //      write(f,#9,RecsEnergiaPorFecha[idx_recs].energia*factor_smec_scada);
  //      Inc(idx_recs);
  //    end
  //    else
  //      write(f,#9,0);
  //  end;
  //
  //  WriteLn(f);
  //end;
  //
  //
  //CloseFile(f);





constructor TParque_ModeloCombinado.CreateLoadFromArchiCalibrador(
  archiCalibrador: string);
begin
  inherited CreateLoadFromArchi(archiCalibrador);
  Calc_MatrizFactores_ME_MT;

end;


constructor TParque_ModeloCombinado.CreateLoadFromArchi(archi: string);
var
  f: textfile;
begin
  setSeparadoresGlobales;
  assignfile(f, archi);
  reset(f);
  inherited CreateLoadFromFile(f);
  MatrizFactor := TMatR.CreateLoadFromFile(f);
  setSeparadoresLocales;
  closefile(f);

end;

procedure TParque_ModeloCombinado.StoreInArchi(archi: string);
var
  f: textfile;
begin
  setSeparadoresGlobales;
  assignfile(f, archi);
  rewrite(f);
  inherited StoreInFile(f);
  MatrizFactor.StoreInFile(f);
  closefile(f);
  setSeparadoresLocales;

end;



function TParque_ModeloCombinado.Calc_PE_ConYSinRO(var PE_Con_RO, PE_Sin_RO: NReal;
  v_anemo, dir_Dato, ro_aire_Dato: NReal; cgm_Dato: NReal): boolean;

var
  pv3Anemo_Dato, pv3Anemo_s, pv3_VientoLibre: NReal;
  convergio: boolean;
  Sector1_Dato, Sector2_Dato: integer;
  peso1_ang, peso2_ang, peso1_tramo, peso2_tramo: NReal;
  Tramo1_Dato, Tramo2_Dato: integer;
  rtramo_dato: NReal;
  ajuste_ME_MT: NReal;
  PE_MT_sin_RO, PE_MT_con_RO: NReal;

begin
  pv3Anemo_Dato := pv3Ofv(v_anemo, ro_aire_Dato);

  parque_MT.Calc_PotMW_init_(dir_Dato);

  if cgm_Dato >= 0.99 * parque_MT.Potencia_Autorizada_MW then
  begin
    PE_MT_con_RO := parque_MT.Calc_PotMW_anemo(ro_aire_Dato, pv3Anemo_Dato,
      cgm_dato, convergio, pv3Anemo_s, pv3_VientoLibre);
    PE_MT_sin_RO := PE_MT_con_RO;
  end
  else
  begin
    PE_MT_con_RO := parque_MT.Calc_PotMW_anemo(ro_aire_Dato, pv3Anemo_Dato,
      cgm_dato, convergio, pv3Anemo_s, pv3_VientoLibre);
    PE_MT_sin_RO := parque_MT.Calc_PotMW_pvLibre(ro_aire_Dato,
      pv3_VientoLibre, parque_MT.Potencia_Autorizada_MW);
  end;


  parque_ME.SectorOfAng(Sector1_Dato, Sector2_Dato, peso1_ang, peso2_ang, dir_Dato);
  //JFP: tengo alguna duda q haya quedado bien esta funcion.
  rtramo_dato := parque_ME.Representantes_pv3_medio.inv_interpol(pv3Anemo_Dato);

  if rtramo_dato <= 1 then
    ajuste_ME_MT := peso1_ang * MatrizFactor.e(Sector1_Dato, 1) +
      peso2_ang * MatrizFactor.e(Sector2_Dato, 1)
  else if rtramo_dato >= parque_ME.Representantes_pv3_medio.n then
    ajuste_ME_MT := peso1_ang * MatrizFactor.e(Sector1_Dato,
      parque_ME.Representantes_pv3_medio.n) + peso2_ang *
      MatrizFactor.e(Sector2_Dato, parque_ME.Representantes_pv3_medio.n)
  else
  begin
    Tramo1_Dato := trunc(rtramo_dato);
    Tramo2_Dato := Tramo1_Dato + 1;
    peso2_tramo := frac(rtramo_dato);
    peso1_tramo := 1 - peso2_tramo;
    ajuste_ME_MT := peso1_ang *
      (peso1_tramo * MatrizFactor.e(Sector1_Dato, Tramo1_Dato) +
      peso2_tramo * MatrizFactor.e(Sector1_Dato, Tramo2_Dato)) +
      peso2_ang * (peso1_tramo * MatrizFactor.e(Sector2_Dato, Tramo1_Dato) +
      peso2_tramo * MatrizFactor.e(Sector2_Dato, Tramo2_Dato));
  end;

  PE_Con_RO := Min(Ajuste_ME_MT * PE_MT_con_RO, parque_MT.Potencia_Autorizada_MW);
  PE_Sin_RO := Min(Ajuste_ME_MT * PE_MT_sin_RO, parque_MT.Potencia_Autorizada_MW);
end;


procedure TParque_ModeloCombinado.Calcular_PE(archi_series: string; dtDesde,
  dtHasta, dtDesdeCalcRed, dtHastaCalcRed: TDateTime; factor_smec_scada: NReal;
  var RecsEnergia10m: TDAofRecsEnergiaPorFecha);
var
  kDato, NDatos: integer;
  series: TSeriesDeDatos;
  dtPrimerMuestra, dtEntreMuestras, dt: TDateTime;

  sVel_mps, sDir_gr, sPot_MW, sTemp_grC, sPre_hPa, sHumRel_p100, sCGM, sDisp: TVectR;

  PE_Dato, PE_ME, dir_Dato, pv3Anemo_Dato, ro_aire_Dato, vel_Dato: NReal;
  Temp_grC_Dato, Pre_hPa_Dato, HumRel_p100_Dato, Peso_Dato, cgm_dato: NReal;
  ajuste_CP: NReal;
  kbuff, NDato_ini, NDato_fin, cant_datos_OK_buffer, kDato_aux: integer;
  PE_MC, num, den, cgm_umbral, err2: NReal;
  Buff_PE_MC, Buff_PE_Dato: TVectR;
  f: textfile;
  PE_MC_sin_RO, PE_MC_con_RO, max_fd: NReal;
  convergio: boolean;
  horas_buffer: integer;
  etha_meteo, f_disp_Dato: NReal;
  ERed_SinAjuste, ERed_ConAjuste: NReal;
  pRed_SinAjuste, pRed_ConAjuste: NReal;
  archisal: string;

label
  ErrorEnInputs;

begin

  series := TSeriesDeDatos.CreateFromArchi(archi_series);

  NDatos := series.series[0].n;

  dtPrimerMuestra := series.dtPrimeraMuestra;

  dtDesde := max(dtDesde, dtPrimerMuestra);
  dtHasta := min(dtHasta, series.dtMuestra(NDatos));

  dtEntreMuestras := series.dtEntreMuestras;

  NDato_ini := trunc((dtDesde - dtPrimerMuestra) / dtEntreMuestras) + 1;
  NDato_fin := trunc((dtHasta - dtPrimerMuestra) / dtEntreMuestras) + 1;

  sVel_mps := series.series[0];
  sDir_gr := series.series[1];
  sPot_MW := series.series[2];

  sTemp_grC := series.series[3];
  sPre_hPa := series.series[4];
  sHumRel_p100 := series.series[5];
  sCGM := series.series[6];
  sDisp := series.series[7];


  max_fd := sDisp.maxVal;
  if max_fd > 1 then
  begin
    writeln('WARNING!!!: max_fd: ' + FloatToStr(max_fd) +
      ' Asumo que es cantidad de máquinas y divido por este valor.');
    sDisp.PorReal(1.0 / max_fd);
  end;


  cgm_umbral := 0.99 * parque_MT.Potencia_Autorizada_MW;

  for horas_buffer := 24 to 24 do
  begin
    archiSal := CarpetaResultados_modelado_RO(parque_MT.nid_central) +
      '\ResultadosMC_' + DateTimeToAAAAMMDDHHMM(dtDesde) + DateTimeToAAAAMMDDHHMM(
      dtHasta) + '.xlt';

    assignfile(f, archiSal);
    rewrite(f);
    Write(f, 'dt');
    Write(f, #9, 'Vel[m/s]');
    Write(f, #9, 'Dir[gr]');
    Write(f, #9, 'P_MW_Dato');
    Write(f, #9, 'CGM_Dato');
    //  write(f, #9, 'P_MW_MC_con_RO' );
    Write(f, #9, 'P_MW_MC_conAjCP');
    //  write(f,#9, 'P_MW_MC_con_RO_sin_Aj' );
    //Write(f, #9, 'P_MW_MC_sinAjCP');
    //Write(f, #9, 'C_ajuste');
    //Write(f, #9, 'EthaMeteo[p.u.]');
    //Write(f, #9, 'f_disp[p.u.]');
    Write(f, #9, 'pRed_conAjCP[MW]');
    //Write(f, #9, 'pRed_sinAjCP[MW]');
    writeln(f);


    kbuff := 0;
    err2 := 0;
    Buff_PE_Dato := TVectR.Create_Init(horas_buffer * 6);
    Buff_PE_Dato.Ceros;
    Buff_PE_MC := TVectR.Create_Init(horas_buffer * 6);
    Buff_PE_MC.Ceros;


    //BUsco datos válidos antes de NDato_ini para inicializar buffer de ajuste de CP.
    kDato := NDato_ini - 1;
    cant_datos_OK_buffer := 0;

    while (kDato >= 1) and (cant_datos_OK_buffer < (6 * horas_buffer)) do
    begin

      dir_Dato := sDir_gr.e(kDato);
      vel_Dato := sVel_mps.e(kDato);
      PE_Dato := sPot_MW.e(kDato);

      Temp_grC_Dato := sTemp_grC.e(kDato);
      Pre_hPa_Dato := sPre_hPa.e(kDato);
      HumRel_p100_Dato := sHumRel_p100.e(kDato);
      cgm_dato := sCGM.e(kDato);

      if ((cgm_dato >= cgm_umbral) and (dir_Dato > 0) and (vel_Dato > 0) and
        (PE_Dato > 0) and (Temp_grC_Dato > 0) and (Pre_hPa_Dato > 0) and
        (HumRel_p100_Dato > 0)) then
        Inc(cant_datos_OK_buffer);

      Dec(kDato);

    end;


    ERed_SinAjuste := 0;
    ERed_ConAjuste := 0;

    kDato_aux := kDato + 1;

    for kDato := kDato_aux to NDato_fin do
    begin

      dir_Dato := sDir_gr.e(kDato);
      vel_Dato := sVel_mps.e(kDato);
      PE_Dato := sPot_MW.e(kDato);

      f_disp_Dato := sDisp.e(kDato);

      Temp_grC_Dato := sTemp_grC.e(kDato);
      Pre_hPa_Dato := sPre_hPa.e(kDato);
      HumRel_p100_Dato := sHumRel_p100.e(kDato);
      ro_aire_Dato := ro_aire(Temp_grC_Dato, Pre_hPa_Dato, HumRel_p100_Dato);
      pv3Anemo_Dato := pv3Ofv(vel_Dato, ro_aire_Dato);
      cgm_dato := sCGM.e(kDato);

      if (cgm_dato < 0) or (dir_Dato < 0) or (vel_Dato < 0) or (PE_Dato < 0) or
        (Temp_grC_Dato < 0) or (Pre_hPa_Dato < 0) or (HumRel_p100_Dato < 0) then
      begin
        PE_MC := -111111;
        PE_MC_con_RO := -111111;
        PE_MC_sin_RO := -111111;
        ajuste_CP := 1;
        goto ErrorEnInputs;
      end;

      convergio := Calc_PE_ConYSinRO(PE_MC_Con_RO, PE_MC_Sin_RO,
        vel_Dato, dir_Dato, ro_aire_Dato, cgm_Dato);
      etha_meteo := parque_MT.Meteo_etha_dir;


      if kbuff >= 1 then
      begin
        num := Buff_PE_Dato.SumaDesdeHasta(1, horas_buffer * 6);
        den := Buff_PE_MC.SumaDesdeHasta(1, horas_buffer * 6);
        if den <= 0.0001 then
          ajuste_CP := 1
        else
          ajuste_CP := num / den;
      end
      else
        ajuste_CP := 1;



      if cgm_dato >= cgm_umbral then
      begin
        kbuff := (kbuff mod (horas_buffer * 6)) + 1;
        Buff_PE_Dato.pon_e(kbuff, PE_Dato);
        Buff_PE_MC.pon_e(kbuff, PE_MC_Sin_RO);
      end;


      ErrorEnInputs:

        if kDato >= NDato_ini then
        begin
          dt := dtPrimerMuestra + (kDato - 1) * dtEntreMuestras;
          Writeln(DateTimeToStr(dt));
          Write(f, dt);
          Write(f, #9, sVel_mps.e(kDato));
          Write(f, #9, dir_Dato);
          Write(f, #9, PE_Dato);
          Write(f, #9, cgm_dato);
          //        Write(f, #9, min(PE_MC_con_RO * ajuste_CP,min(cgm_dato,parque_MT.Potencia_Autorizada_MW)));
          Write(f, #9, min(PE_MC_sin_RO * ajuste_CP, parque_MT.Potencia_Autorizada_MW));
          //        Write(f, #9, PE_MC_con_RO);
          //Write(f, #9, PE_MC_sin_RO);
          //Write(f, #9, ajuste_CP);
          //Write(f, #9, etha_meteo);
          //Write(f, #9, f_disp_Dato);

          if (dt >= dtDesdeCalcRed) and (dt <= dtHastaCalcRed) and
            (cgm_dato > -1) and
            (cgm_dato < (self.parque_MT.Potencia_Autorizada_MW - 0.5)) and
            (PE_Dato > (cgm_dato - 0.03*parque_MT.Potencia_Autorizada_MW)) and
            (PE_Dato < (cgm_dato + 0.03*parque_MT.Potencia_Autorizada_MW)) then
          begin
            pRed_SinAjuste := max(PE_MC_sin_RO - PE_Dato, 0);
            pRed_ConAjuste := max(
            0, (min(PE_MC_sin_RO * ajuste_CP, parque_MT.Potencia_Autorizada_MW) - PE_Dato));

            if pRed_ConAjuste>0 then
              AddItem(dt,pRed_ConAjuste/6,RecsEnergia10m);

            ERed_SinAjuste := ERed_SinAjuste + pRed_SinAjuste / 6.0;
            ERed_ConAjuste := ERed_ConAjuste + pRed_ConAjuste / 6.0;
          end
          else
          begin
            pRed_SinAjuste := 0;
            pRed_ConAjuste := 0;
          end;

          Write(f, #9, pRed_ConAjuste);
          //Write(f, #9, pRed_SinAjuste);

          writeln(f);
          err2 := err2 + sqr(PE_MC_sin_RO * ajuste_CP - PE_Dato);
        end;
    end;

    writeln(f);
    writeln(f,'Fecha inicio',#9,'Fecha fin', #9, ' Energía no suministrada SCADA [MWh]',#9, ' Factor SMEC/SCADA [p.u] ', #9, ' Energía no suministrada SMEC [MWh]');
    writeln(f,FormatDateTime('dd-mm-yyyy hh:mm',dtDesdeCalcRed)    ,#9,  FormatDateTime('dd-mm-yyyy hh:mm',dtHastaCalcRed-1/(24*6)) ,#9,ERed_ConAjuste,#9,factor_smec_scada,#9,ERed_ConAjuste*factor_smec_scada);
    //writeln(f, ERed_SinAjuste, #9, '[MWh] Eenergía Reducida Sin AjCP');

    //WriteLn('Horas BUffer= ', horas_buffer, '// Error Cuad: ', err2);

    Buff_PE_Dato.Free;
    Buff_PE_MC.Free;

    closefile(f);

  end;

  WriteLn('Fin cálculo del Modelo Combinado');
end;




procedure TParque_ModeloCombinado.Calc_MatrizFactores_ME_MT;

var
  kAng, kTramoVel, cnt_iter, Max_cnt_iter, i, j: integer;
  pv3, PE_MW_Modelo, PE_MW_Medida: NReal;
  ang: NReal;
  pv3Anemo, pv3Anemo_s: NReal;
  vpe, vPeso: TVectR;
  peso, epsilon: NReal;

  //Matriz que tiene los coeficientes ME/MT solo para los clusters
  //con peso> UmbralPoblacionMinimaCluster. El resto de los valores es 0.
  MatrizBorde: TMatR;

begin
  MatrizBorde := TMatR.Create_Init(parque_ME.NSectores, parque_ME.NTramosVel);
  MatrizBorde.Ceros;

  MatrizFactor := TMatR.Create_Init(parque_ME.NSectores, parque_ME.NTramosVel);
  for i := 1 to parque_ME.NSectores do
    for j := 1 to parque_ME.NTramosVel do
      MatrizFactor.pon_e(i, j, 1);

  CalcularMatrizBorde(MatrizBorde);

  Max_cnt_iter := 10000;

  cnt_iter := 0;
  Barrido_ImponerBorde(MatrizFactor, MatrizBorde);
  repeat
    epsilon := Barrido_PromediarCasilleros(MatrizFactor, MatrizBorde);
    writeln('cnt_iter: ', cnt_iter, ', epsilon: ', epsilon);
    cnt_iter := cnt_iter + 1;
  until ((cnt_iter > Max_cnt_iter) or (epsilon < 1e-4));

  //MatrizFactor.WriteArchiXLT('c:\basura\MatrizFactor.xlt');
  MatrizBorde.Free;
end;

procedure TParque_ModeloCombinado.CalcularMatrizBorde(var MatrizBorde: TMatR);
var
  kAng, kTramoVel: integer;
  pv3, PE_MW_Modelo, PE_MW_Medida: NReal;
  ang: NReal;
  pv3Anemo, pv3Anemo_s, pv3_VientoLibre: NReal;
  vpe, vPeso: TVectR;
  peso: NReal;
  convergio: boolean;
begin
  for kAng := 1 to parque_ME.NSectores do
  begin
    writeln('kAng: ', kang, ' **********************');
    vPeso := parque_ME.peso_Representantes.fila(kang);
    vpe := parque_ME.Representante_FO.fila(kang);
    ang := parque_ME.AngOfjSector(kAng);

    parque_MT.Calc_PotMW_init_(ang);
    for kTramoVel := 1 to parque_ME.NTramosVel do
    begin
      Write(kTramoVel, ', ');
      peso := vPeso.e(kTramoVel);
      if peso < UmbralPoblacionMinimaCluster then
        continue;
      pv3Anemo := parque_ME.Representantes_pv3_medio.e(kTramoVel);

      PE_MW_Medida := parque_ME.Calc_PE(ang, pv3Anemo, ro_BASE, peso);

      PE_MW_Modelo := parque_MT.Calc_PotMW_anemo(ro_BASE, pv3Anemo,
        parque_MT.Potencia_Autorizada_MW, convergio, pv3Anemo_s, pv3_VientoLibre);
      if abs(PE_MW_Modelo) > 1 then
        MatrizBorde.pon_e(kAng, kTramoVel, PE_MW_Medida / PE_MW_Modelo);
    end;
    writeln;

  end;

end;

procedure TParque_ModeloCombinado.Barrido_ImponerBorde(var MatrizFactor: TMatR;
  MatrizBorde: TMatR);
var
  Nfilas, Ncolumnas: integer;
  fila, col: integer;

begin
  Nfilas := MatrizFactor.nf;
  Ncolumnas := MatrizFactor.nc;

  for fila := 1 to Nfilas do
    for col := 1 to Ncolumnas do
    begin
      if MatrizBorde.e(fila, col) > 0 then
        MatrizFactor.pon_e(fila, col, MatrizBorde.e(fila, col));
    end;

end;

function TParque_ModeloCombinado.Barrido_PromediarCasilleros(var MatrizFactor: TMatR;
  MatrizBorde: TMatR): NReal;
var

  m, n: integer;
  k, j: integer;
  mx, my: TMatR;

begin

  m := MatrizFactor.nf;
  n := MatrizFactor.nc;

  mx := TMatR.Create_Clone(MatrizFactor);
  my := MatrizFactor;

  for k := 1 to m do
    for j := 1 to n do
      if MatrizBorde.e(k, j) <= 0 then
      begin
        if (k > 1) and (j > 1) and (k < m) and (j < n) then
        begin
          my.pon_e(k, j,
            (mx.e(k, j - 1) + mx.e(k, j + 1) + mx.e(k - 1, j) + mx.e(k + 1, j)) / 4);
        end
        else
        if k = 1 then
        begin
          if j = 1 then
            my.pon_e(k, j,
              (mx.e(k, j) + mx.e(k, j + 1) + mx.e(m, j) + mx.e(k + 1, j)) / 4)
          else if j = n then
            my.pon_e(k, j,
              (mx.e(k, j - 1) + mx.e(k, j) + mx.e(m, j) + mx.e(k + 1, j)) / 4);
        end
        else // k = m
        begin
          if j = 1 then
            my.pon_e(k, j,
              (mx.e(k, j) + mx.e(k, j + 1) + mx.e(k - 1, j) + mx.e(1, j)) / 4)
          else if j = n then
            my.pon_e(k, j,
              (mx.e(k, j - 1) + mx.e(k, j) + mx.e(k - 1, j) + mx.e(1, j)) / 4);
        end;
      end;

  mx.PorReal(-1);
  mx.Suma(mx, my);

  Result := abs(mx.MaxVal);

  mx.Free;

end;



end.






