unit uSombraGenERNC;

{$mode delphi}

interface

uses
  Classes, SysUtils,  Math, xmatdefs, matreal, matent, uauxiliares, umodelosintcegh;


const
  // constantes para Base de la PViento.
  Base_Velocidad_mps = 14.0;
  Base_Temperatura_grC = 18.0;
  Base_Presion_hPA = 997.0;
  Base_HumedadRelativa_p100 = 73.0;


type

//Se realiza una aproximación a los datos reales Pe vs Pv, ajustando la curva de Pe vs Pv del
//fabricante del aerogenerador junto con la matriz de correlaciones de sombra del parque.
//f_salida = f_teo_molino_1*a_1 + f_teo_molino_2*a_2 + ...+ f_teo_molino_n*a_n
// Pe = f_salida(Pv_reales)



TParqueEolicoAjustado = class
  nombre: string; // nombre del generador
  base_PV: NReal; // ro_ v^3 tomada com base
  base_PE: NReal; // MW tomados como base.

  NPuntosD: integer; // Cantidad de Puntos de Dirección.
  OverlappingD: NReal; // solapamiento

  NPuntosP: integer; // Cantidad de Puntos en las curvas de potencias.
  PV: TDAOfVectR; // Potencias del Viento en p.u.
  PE: TDAOfVectR; // Potencias Eleéctricas en p.u.
  cntMuestras: TDAOfVectE;


  // Crea una estructura vacía.
  constructor Create(
    nombre: string;
    NPuntosD: integer;
    OverlappingD: NReal;
    NPuntosP: integer;  base_PE: NReal );

  // Destruye la instancia.
  procedure Free;


  // Cargar modelo del fabricante del aerogenerador.
  procedure Loadmodelofab( archi: string );

  //
  procedure CalcularCoefas( archi: string );

  //
  procedure AproximarCurvas( archi: string );


  // Carga el modelo desde disco.
  constructor ReadFromArchi( archi: string );

  // Guarda el modelo en disco
  constructor WriteToArchi( archi: string );

  //estima la potencia eolica generada a partir de las medidas de viento de la Estación Meteo. del parque.
  procedure EstimarPotenciaEolica;



private
  series1, series_a, series_b, series3, series4, series5: TSeriesDeDatos;
  dang: NReal;


end;



implementation

// Crea una estructura vacía.
constructor TParqueEolicoAjustado.Create(
  nombre: string;
  NPuntosD: integer;
  OverlappingD: NReal;
  NPuntosP: integer;  base_PE: NReal );
var
  k: integer;
begin
  inherited Create;
  dang:= 360.0 / NPuntosD;
  self.Nombre:= nombre;
 // base_PV:= ro_aire( Base_Temperatura_grC, Base_Presion_hPA, Base_HumedadRelativa_p100 ) * power( Base_Velocidad_mps, 3 );
  base_PV:=1;
  self.base_PE:= base_PE;
  self.NPuntosD:= NPuntosD;
  self.OverlappingD:= OverlappingD;
  self.NPuntosP:= NPuntosP; // Cantidad de Puntos en las curvas de potencias.
  setlength( PV, NPuntosD );
  setlength( PE, NPuntosD );
  setlength( cntMuestras, NPuntosD );
  for k:= 0 to NPuntosD-1 do
  begin
    PV[k]:= TVectR.Create_Init( NPUntosP );
    PE[k]:= TVectR.Create_Init( NPUntosP );
    cntMuestras[k]:= TVectE.Create_Init( nPuntosP );
  end;
  series:= nil;
end;


procedure TParqueEolicoAjustado.Free;
var
  k: integer;
begin
  for k:= 0 to NPuntosD-1 do
  begin
    PE[k].Free;
    PV[k].Free;
    cntMuestras[k].Free;
  end;
  setlength( PE, 0 );
  setlength( PV, 0 );
  setlength( cntMuestras, 0 );
  if series <> nil then
  begin
    series.Free;
    series:= nil;
  end;
  inherited Free;
end;

function ro_aire(Temp_grC, Pre_hPa, HumRel_p100: NReal): NReal;
begin
  Result := (0.34848 * Pre_hPa - 0.009 * HumRel_p100 * exp(0.061 * Temp_grC)) /
    (273.16 + Temp_grC);
end;

procedure TParqueEolicoAjustado.LoadCurvaModeloAero(Temp,Pre,Hum,base_PV:NReal,archi: string ):TVectR;
var Temp,Pre, Hum, base_PV:NReal;
    Vel,PotElectrica, PotViento,salida:TVectR;

begin

  {Temp:=0;
  Pre:=0;
  Hum:=0;
  base_PV:=1;}
  salida:=TVectR.Create_Init(2);
  series1 := TSeriesDeDatos.CreateFromArchi( archi );

  Vel:=series1.series[0];
  salida.pon_e(2,series1.series[1]);

  fro := ro_aire(Temp, Pre, Hum) / base_PV;

  for j:=0 to series1.NPuntos do begin

        PotViento.pon_e(j, fro * power(Vel.e(j), 3.0));
  end;
     salida.pon_e(1,PotViento);
     result:=salida;
end;

procedure TParqueEolicoAjustado.LoadCurvaSombra(archi: string);
begin
  series2 := TSeriesDeDatos.CreateFromArchi( archi );
end;

procedure TParqueEolicoAjustado.LoadCurvaEstimadaParque(angulo:NReal);
begin
  series3 := TSeriesDeDatos.CreateFromArchi( 'c:\basura\Modelo_Parque\resdir_' + Nombre + '_' + angulo + '.xlt' );
end;


function TParqueEolicoAjustado.RangoParaAs (VelocidadViento:NReal):NReal;

var
  curvarriba,curvabajo:TVectR;
  rango:NReal;

begin

  curvarriba:=series2.series[0];
  curvabajo:=series2.series[1];

  Valarriba:=curvarriba.interpol(VelocidadViento);
  Valabajo:=curvabajo.interpol(VelocidadViento);

  rango:=Valarriba-Valabajo;

  result:=rango;

end;

procedure TParqueEolicoAjustado.MinimizarCoefA (dang:NReal, archi:string);
var
  CoefA,rangoVlibre,Vaero1,Paero1,Vaero2,Paero2,indAero2,indParque1,indParque2,Pparque1,Pparque2:NReal;
  diff, iter,x,z:integer;
  ModeloParque, ModeloAero,errorP,diff:TVectR;
begin

 ModeloParque := TSeriesDeDatos.CreateFromArchi( 'c:\basura\Modelo_Parque\resdir_' + Nombre + '_' + dang + '.xlt' );

 ModeloAero:= LoadCurvaModeloAero(lugar_del_archivo);

  CoefA:=0;
  iter:=0;
  diff:=1000;

 while (diff>1)or(iter<100) do
begin
       CoefA:=Coefa+0.1;

       for x:=0 to   ModeloAero.NPuntos do
 begin

        rangoVlibre:=RangoParaAs(ModeloAero[1].e(x));
        Vaero1:=ModeloAero[1].e(x);
        Paero1:=ModeloAero[2].e(x);

        Vaero2:= CoefA*rangoVlibre + Vaero1;
        indAero2:=ModeloAero.inv_interpol(Vaero2);
        Paero2:=ModeloAero.e(indAero2);

        indParque1:=ModeloParque[1].interpol(Vaero1);
        indParque2:=ModeloParque[1].interpol(Vaero2);
        Pparque1:=ModeloParque[1].inv_interpol(indParque1);
        Pparque2:=ModeloParque[1].inv_interpol(indParque2);

        errorP.pon_e(x,Pparque2-Paero2);

 end;
       diff:=0;

       for z:=0 to length(errorP) do
       begin
       diff:=diff+errorP.e(z);
       end;

       diff:=diff/z;
end;

      iter:=iter+1;

end;



constructor TParqueEolicoAjustado.ReadFromArchi( archi: string );
begin

end;

constructor TParqueEolicoAjustado.WriteToArchi( archi: string );
begin

end;

procedure TParqueEolicoAjustado.EstimarPotenciaEolica;
begin

end;



end.

