{+doc
+NOMBRE: ucalibrarconopronosticos
+CREACION: 2011-11-22
+AUTORES: rch
+REGISTRO:
+TIPO: Unidad Pascal.
+PROPOSITO:procedimiento de calibración de un cono de pronosticos
+PROYECTO:SimSEE

+REVISION:
+AUTOR:
+DESCRIPCION:
-doc}
unit ucalibrarconopronosticos;

{$mode delphi}

interface

uses
{$IFDEF CALIBRAR_PRONOSTICOS_TRAZOSXY}
  Graphics, utrazosxy,
{$ENDIF}
  uauxiliares, Math, Dialogs,
  umodelosintcegh, upronostico, xmatdefs, ufechas,
  autoesca,
  Classes, SysUtils, matreal, uFuenteSintetizador, usalasdejuego;


(*+doc
Este procedimiento recibe un modelo CEGH y una descripción de los pronosticos asociados
a sus bornes de salida tal como es utilizado porp las FuentesSintetizador.
Si bien los pronosticos pueden especificar la Probabilidad de Excedencia de la GUIA este
procedimiento ignora dicha PE y calcula los vectores "sesgos" y "factor" para lograr
un cono centrado sobre la guía. Se entiende por "Centrado" que la guía tenga probabilidad
de excedencia 50% en el cono.
-doc*)
procedure CalibrarConoCentrado(cegh: TModeloSintetizadorCEGH;
  pronosticos: TPronosticos; fechaIniSim: TFecha);


type
(*+doc
Esta clase permite calcular las series de un cono de pronóstico dado un modelo CEGH, un pronótico para una fecha
de inicio. La determinación del cono se hace mediante una simulación de la salida del CEGH de la cantidad de
crónicas especificadas en NCronicas. La simulación se realiza desde FechaIniSim y se finaliza en FechaFinSim
-doc*)
  TSeriesConoPronostico = class
    NPasosT: integer;

    speInf, spe50, speSup: TMatR;
    sVE: TMatR;
    fcegh: TFuenteSintetizadorCEGH; // referencia a la fuente de creación

    pe_inf, pe_sup: NReal;

    constructor CreateFromSim(fcegh: TFuenteSintetizadorCEGH;
      fechaIniSim, fechaFinSim: TFecha; NCronicas: integer; pe_inf, pe_sup: NReal);
    procedure Free;
    {$IFDEF CALIBRAR_PRONOSTICOS_TRAZOSXY}
    procedure plot(Sender: TComponent);
    {$ENDIF}
  end;




implementation

// extiende con CEROS las guias de pronósticos en espacios gaussianos.
// la hipótesis es que luego de finalizado el cono, la guía es el CERO
// en el espacio gaussiano.
function fguia_extendida(guia_eg: TDAOfNreal; jPaso: integer): NReal;
begin
  if jPaso < length(guia_eg) then
    Result := guia_eg[jPaso]
  else
    Result := 0;
end;




procedure CalibrarConoCentrado(cegh: TModeloSintetizadorCEGH;
  pronosticos: TPronosticos; fechaIniSim: TFecha);
var
  k: integer;
  ydet, xdet: NReal;

  fecha: TFecha;

  NSeries: integer;
  kserie_: integer;
  kpaso: integer;

  a: NReal;
  jserie, jbase: integer;
  kretardo: integer;

  ifil, jcol: integer;
  p: array of TPronostico; // solo para facilidad de escritura.

  j: integer;
  kselector: integer;
  delta_sesgo: NReal;

  kEscenario, jEscenario, miSerie: integer;
  peso: NReal;

  // funcion auxiliar para asignar CERO si me voy por fuera de la guia
  function s(kserie, jpaso: integer): NReal;
  begin
    if jpaso <= high(p[kserie].guia_eg) then
      Result := p[kserie].guia_eg[jpaso]
    else
      Result := 0;
  end;

begin
  NSeries := cegh.nBornesSalida;

  // apuntamos los p a los pronosticos para facilitar referencia.
  setlength(p, Pronosticos.Count);
  for k := 0 to high(p) do
    p[k] := pronosticos[k] as TPronostico;


(*** MMMM para mi que esto no estaba bien
mejor que cada vez vuelva a sincronizar
  // revisamos las fechas de inicio de los pronósticos.
  // si son "El Origen de los Tiemps, dt = 0 le imponemos la
  // fecha de inicio de la simulación.
  for k := 0 to high(p) do
  begin
    if p[k].fechaIniSesgo_.dt = 0 then
    begin
      p[k].fechaIniSesgo_.dt := fechaIniSim.dt;
      p[k].calcular_rangos(cegh.durPasoDeSorteoEnHoras);
    end;
  end;
  ******)
  for k := 0 to high(p) do
  begin
    p[k].fechaIniSesgo_.dt := fechaIniSim.dt;
    p[k].calcular_rangos(cegh.durPasoDeSorteoEnHoras);
  end;


  // ajustamos dimensiones y limpiamos sesgos y factor
  for k := 0 to high(p) do
  begin
    setlength(p[k].sesgo, p[k].NPCC + p[k].NPLC);
    vclear(p[k].sesgo);
    setlength(p[k].factor, p[k].NPSA + p[k].NPAC);
    vclear(p[k].factor);
  end;

  // llenamos los factores de apertura del cono
  // hasta NPSA dejamos los ceros y a partir de ahí abrimos el cono aumentando
  // linealmente el factor.
  for k := 0 to high(p) do
  begin
    for j := 0 to p[k].NPSA - 1 do
      p[k].factor[j] := 0.0;
    for j := p[k].NPSA to high(p[k].factor) do
    begin
      p[k].factor[j] := ( j - p[k].NPSA ) / p[k].NPAC;
    end;
  end;

  // ahora creamos la imagen de las guías de los pronósticos en el espacio
  // gaussiano.
  for k := 0 to high(p) do
  begin
    setlength(p[k].guia_eg, length(p[k].guia));
    fecha := TFecha.Create_Clone(fechaIniSim);

    // retrocedemos para la cantidad de retardos por la duración del paso de sorteo.
    fecha.addHoras(-p[k].NRetardos * cegh.durPasoDeSorteoEnHoras);

    for kpaso := 0 to high(p[k].guia_eg) do
    begin
      ydet := p[k].guia[kpaso];
      kselector := cegh.kSelectorDeformador(fecha);

      kserie_:= p[k].kSerie;

      xdet := cegh.xTog(ydet, kserie_ + 1, kselector + 1);
      p[k].guia_eg[kpaso] := xdet;
      fecha.addHoras(cegh.durPasoDeSorteoEnHoras);
    end;
    fecha.Free;
  end;



  // ahora, proyectamos la salida del filtro para cada serie conociendo
  // los estados anteriores y la diferencia entre el proyectado y el valor
  // siguiente en los determinismos gaussianos es el sesgo en la fuente.
  for kEscenario := 0 to high( p)  do
  begin
    jbase := p[kEscenario].NRetardos - 1;

    if cegh.A_cte = nil then
      // Necesita calcular kselector para seleccionar matriz mcA
    begin
      // neceisto la fecha para calcular el kSelector
      fecha := TFecha.Create_Clone(fechaIniSim);
      // retrocedemos para la cantidad de retardos por la duración del paso de sorteo.
      fecha.addHoras(-p[kEscenario].NRetardos * cegh.durPasoDeSorteoEnHoras);
    end;


    for kpaso := 0 to p[kEscenario].NPCC - 1 do
    begin
      if cegh.A_cte = nil then
      // Necesita calcular kselector para seleccionar matriz mcA
      begin
        kselector := cegh.kSelectorDeformador(fecha);
        fecha.addHoras(cegh.durPasoDeSorteoEnHoras);
      end;

      miSerie:= p[kEscenario].kSerie;
      ifil := miSerie + 1;

      a := 0;
      for kretardo := 0 to p[kEscenario].NRetardos - 1 do
      begin
        for jEscenario:= 0 to high(p) do
        begin
          jserie:= p[jEscenario].kSerie;

          if jserie <> miSerie then
             peso:= p[jEscenario].p
          else
            if jEscenario = kEscenario then
             peso:=1
            else
              peso:= 0;
          jcol := kretardo * NSeries + jserie + 1;
          xdet := fguia_extendida(p[kEscenario].guia_eg, jbase - kretardo);
          if cegh.A_cte <> nil then
            a := a + cegh.A_cte.e(ifil, jcol) * xdet * peso
          else
            a := a + cegh.mcA[kSelector].e(ifil, jcol) * xdet * peso;
        end;
      end;

      p[kEscenario].sesgo[kpaso] := p[kEscenario].guia_eg[jbase + 1] - a;
      Inc(jbase);
    end;

    // si está especificado Número de Pasos de Liberación del Cono
    // llevamos los sesgos a CERO en forma lineal.
    if (p[kEscenario].NPLC > 0) and (p[kEscenario].NPCC > 0) then
    begin
      delta_sesgo := p[kEscenario].sesgo[p[kEscenario].NPCC - 1] / p[kEscenario].NPLC;
      for kpaso := p[kEscenario].NPCC to high(p[kEscenario].sesgo) do
        p[kEscenario].sesgo[kpaso] := p[kEscenario].sesgo[kpaso - 1] - delta_sesgo;
    end;

    if cegh.A_cte = nil then
      fecha.Free;

  end;

  pronosticos.WriteToArchi('pronosticos_calibrador.txt');
end;




var
  //Es un arreglo de boneras, donde cada bornera es una matriz
  //Las filas de esta matriz son los pasos y las columnas las cronicas
  preview_datos: array of TMatR;
  preview_fuente: TFuenteSintetizadorCEGH;
  preview_kCronica, preview_jPaso, preview_nseries, preview_DimEstadoReducido: integer;

  preview_despBornera: integer; // desplazamiento en la bornera para acceder a la salida

  // del filtro en el mundo real. = nSeries + (nSeries*NRetardos)
  preview_NCronicas, preview_NPasos: integer;
  {$IFDEF CALIBRAR_PRONOSTICOS_TRAZOSXY}
  trx: array of TfrmDllForm;

  {$ENDIF}

procedure preview_inicioCronica;
begin
  preview_jPaso := 1;
  Inc(preview_kCronica);
end;

procedure preview_finpaso;
var
  jBornera, j: integer;
begin
  for jBornera := 0 to preview_nseries - 1 do
  begin
    preview_datos[jBornera].pon_e(preview_jPaso, preview_kCronica,
      preview_fuente.bornera[preview_despBornera + jBornera]);
  end;
  for j := 0 to preview_DimEstadoReducido - 1 do
  begin
    with preview_fuente do
      ReducirEstado(XRed, jPrimer_X_x, datosModelo_Opt, datosModelo_Opt.MRed, Bornera);

    preview_datos[preview_nseries + j].pon_e(preview_jPaso, preview_kCronica,
      preview_fuente.XRed[j]);
  end;
  Inc(preview_jPaso);
end;



constructor TSeriesConoPronostico.CreateFromSim(fcegh: TFuenteSintetizadorCEGH;
  fechaIniSim, fechaFinSim: TFecha; NCronicas: integer; pe_inf, pe_sup: NReal);

var
  fecha: TFecha;
  sala: TSalaDeJuego;
  durpos: TDAOfNReal;
  jPaso, kSerie: integer;
  jpeInf, jpe50, jpeSup: integer;
  vdatos: TVectR;

  kCronica: integer;
  sal: textfile;
  old_dir: string;

begin
  inherited Create;

  old_dir:= getcurrentdir;

  fcegh.WriteToArchi('fcegh_calibrador.txt');


  self.fcegh := fcegh;

  self.pe_inf := pe_inf;
  self.pe_sup := pe_sup;


  setlength(durpos, 1);
  durpos[0] := fcegh.datosModelo_Sim.durPasoDeSorteoEnHoras;

  // creamos un clon para que cuando haga el FREE de la Sala no se me pierda
  // la fuente.
  preview_fuente := fcegh.Create_Clone as TFuenteSintetizadorCEGH;

  sala := TSalaDeJuego.Create(fcegh.capa, 'Calibrador',
    // rango de simulación
    fechaIniSim, fechaFinSim, TFecha.Create_Dt(0),
    // rango de optimización (pongo lo mismo que sim - no importa )
    fechaIniSim, fechaFinSim,
    durpos);

  sala.setDirCorrida('_calibrador_.ese');

  //Inicializamos todas las variables
  preview_NPasos := sala.globs.calcNPasosSim;
  preview_kCronica := 0;

  preview_nseries := fcegh.datosModelo_Sim.nBornesSalida;
  preview_DimEstadoReducido := fcegh.datosModelo_Sim.MRed.nf;

  preview_despBornera := fcegh.dim_RB+ fcegh.dim_Wa + fcegh.datosModelo_Sim.nRetardos *
    preview_nseries;

  NPasosT := preview_NPasos;
  preview_NCronicas := NCronicas;

  speInf := TMatR.Create_Init(preview_NSeries + preview_DimEstadoReducido,
    preview_NPasos);
  spe50 := TMatR.Create_Init(preview_NSeries + preview_DimEstadoReducido,
    preview_NPasos);
  speSup := TMatR.Create_Init(preview_NSeries + preview_DimEstadoReducido,
    preview_NPasos);
  sVE := TMatR.Create_Init(preview_NSeries + preview_DimEstadoReducido,
    preview_NPasos);

  sala.globs.procNot_InicioCronica := preview_inicioCronica;
  sala.globs.procNot_FinPaso := preview_finpaso;


  //Inicializo array para el tamaño de series
  setlength(preview_datos, preview_nseries + preview_DimEstadoReducido);
  //Inicializo matriz como num de pasos x num de cronicas
  for kSerie := 0 to high(preview_datos) do
  begin
    preview_datos[kSerie] := TMatR.Create_Init(preview_NPasos, preview_NCronicas);
  end;

  //Agregamos fuente a la sala
  sala.listaFuentes.Add(preview_fuente);


  (**** OJO hago como que optmicé para que exista globs.CF ****)
  sala.inicializarOptimizacion_subproc01;
  sala.inicializarOptimizacion_subproc02(nil, nil);



  sala.globs.abortarSim := False;
  sala.globs.NCronicasSim := NCronicas;
  sala.globs.semilla_inicial_sim := 31;
  // !!!!!!!!!!!!! SIMULAR !!!!!!!!!!!!!!!!!!
  setSeparadoresGlobales; // para que escriba en global
  sala.Simular(0, True);
  setSeparadoresLocales;
  // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  jpeInf := min(trunc(pe_inf * NCronicas) + 1, NCronicas);
  jpe50 := min(trunc(0.5 * NCronicas) + 1, NCronicas);
  jpeSup := min(trunc(pe_sup * NCronicas) + 1, NCronicas);


  assignfile(sal, 'ucalibrador.xlt');
  rewrite(sal);
  writeln(sal, 'NSeries: ', preview_NSeries);
  writeln(sal, 'NPasos: ', preview_NPasos);
  writeln(sal, 'DimEstadoReducido: ', preview_DimEstadoReducido);

  for kSerie := 1 to length(preview_datos) do
  begin
    writeln(sal, 'Serie: ', kSerie);
    for jPaso := 1 to preview_NPasos do
    begin
      Write(sal, jPaso);
      //datos[kSerie][jPaso][jCronica]
      vdatos := preview_datos[kSerie - 1].Fila(jPaso);
      for kCronica := 1 to vdatos.n do
        Write(sal, #9, vdatos.e(kCronica));
      writeln(sal);
    end;
    writeln(sal);
  end;

  writeln(sal, '**********CONO RESUMEN*********');
  writeln(sal, 'jpeInf: ', jpeInf, ', jpe50: ', jpe50, ', jpeSup: ', jpeSup);

  //Tenemos que ordenar los vectores de la matriz para poder imprimir el cono
  for jPaso := 1 to preview_NPasos do
  begin
    Write(sal, jPaso);
    for kSerie := 1 to length(preview_datos) do
    begin
      //datos[kSerie][jPaso][jCronica]
      vdatos := preview_datos[kSerie - 1].Fila(jPaso);
      vdatos.Sort(True);

      sVE.pon_e(kSerie, jPaso, vdatos.promedio);
      speInf.pon_e(kSerie, jPaso, vdatos.e(jpeInf));
      spe50.pon_e(kSerie, jPaso, vdatos.e(jpe50));
      speSup.pon_e(kSerie, jPaso, vdatos.e(jpeSup));

      Write(sal, #9, sVE.e(kSerie, jPaso));
      Write(sal, #9, speInf.e(kSerie, jPaso));
      Write(sal, #9, spe50.e(kSerie, jPaso));
      Write(sal, #9, speInf.e(kSerie, jPaso));
    end;
    writeln(sal);
  end;

  closefile(sal);

  sala.Free;
  preview_fuente := nil; // ya la liberó la sala.
  for kSerie := 0 to high(preview_datos) do
    preview_datos[kSerie].Free;
  setlength(preview_datos, 0);

  setcurrentdir( old_dir );
end;

procedure TSeriesConoPronostico.Free;
begin
  speInf.Free;
  spe50.Free;
  speSup.Free;
  sVE.Free;
end;



{$IFDEF CALIBRAR_PRONOSTICOS_TRAZOSXY}
procedure TSeriesConoPronostico.plot(Sender: TComponent);
var
  kSerie, jPaso: integer;
  colores: array of TColor;
  nombreBorne: string;

  kmin, kmax: integer;
  minVal, maxVal, dVal: NReal;

  NDivY: integer;
  guia: TDAOfNReal;
  jPasoGuia, jPasoGuia_Ultimo: integer;

  iserie_VE, iserie_Guia, iserie_pe05, iserie_pe50, iserie_pe95: integer;

begin

  if NPasosT <= 0 then
  begin
    ShowMessage('No hay cono de pronósticos especificado.');
    exit;
  end;

  if length(trx) <> 0 then
    for kSerie := 0 to high(trx) do
      if trx[kSerie] <> nil then
        trx[kSerie].Close;

  SetLength(trx, preview_nseries + preview_DimEstadoReducido);
  //Tamaño del vector de graficos
  //Creo un array con los colores para todas las posibles series
  setlength(colores, 12);
  colores[0] := clBlue;
  colores[1] := clOlive;
  colores[2] := clTeal;
  colores[3] := clGreen;
  colores[4] := clNavy;
  colores[5] := clPurple;
  colores[6] := clBlack;
  colores[7] := clGray;
  colores[8] := clLime;
  colores[9] := clFuchsia;
  colores[10] := clMaroon;
  colores[11] := clRed;

  //Comienza loop de imprimir
  for kSerie := 0 to preview_nseries + preview_DimEstadoReducido - 1 do
  begin

    if kSerie < preview_nseries then
      nombreBorne := fcegh.NombresDeBornes_Publicados[kSerie]
    else
      nombreBorne := fcegh.datosModelo_Sim.nombreVarE[kSerie - preview_nseries];


    minVal := speInf.fila(kSerie + 1).minval;
    maxVal := speSup.fila(kSerie + 1).maxval;

    NDivY := 8;
    Escala125N(minVal, maxVal, dVal, NDivY, 1);

    if trx[kSerie] <> nil then
      trx[kSerie].Close;

    trx[kSerie] := TfrmDllForm.Create(Sender);

    iserie_VE:= 1;
    trx[kSerie].CrearDiagramaXY(
      nombreBorne, // Nombres
      NPasosT,      // MaxNPuntos
      False,       // ciruclar
      'paso',         // nombre_sx
      'VE',  // nombre_sy1: pchar;
      colores[0],  //  color_sy1: TColor;
      1, NPasosT,   //  x1, x2,
      minVal, maxVal, //y1, y2
      10, NDivY        // NDivX, NDivY
      );

    trx[kSerie].nilOnClose := @trx[kSerie];

    trx[kSerie].Show;

    //Creo las series
    if kSerie < preview_nseries then
      iserie_Guia:= trx[kSerie].CrearSerieXY('Guía', NPasosT, False, colores[1],
        TM_Cuadrado, colores[1], colores[1], 3);

    iserie_pe05:= trx[kSerie].CrearSerieXY('pe_' + FloatToStr(trunc(self.pe_inf * 100 + 0.5)) + '%',
      NPasosT, False, colores[2], TM_Triangulo, colores[2], colores[2], 3);
    iserie_pe50:= trx[kSerie].CrearSerieXY('pe_50%', NPasosT, False, colores[3],
      TM_Circulo, colores[3], colores[3], 3);
    iserie_pe95:= trx[kSerie].CrearSerieXY('pe_' + FloatToStr(trunc(self.pe_sup * 100 + 0.5)) + '%',
      NPasosT, False, colores[4], TM_Cruz45, colores[4], colores[4], 3);

    trx[kSerie].dbj_gridx;
    trx[kSerie].dbj_gridy;
    trx[kSerie].dbj_borde;
    trx[kSerie].titulo(nombreBorne);
    trx[kSerie].etiquetas_x(0, NPasosT);
    trx[kSerie].etiquetas_y(MinVal, maxVal);
    trx[kSerie].xlabel('PasoT');
    //    trx[kSerie].ylabel('M^3/s');


    if kSerie < preview_nseries then
    begin
      guia := TPronostico(fcegh.pronosticos[kSerie]).guia;
      jPasoGuia := TPronostico(fcegh.pronosticos[kSerie]).NRetardos - 1;
      jPasoGuia_Ultimo := jPasoGuia + TPronostico(fcegh.pronosticos[kSerie]).NPCC;
    end;

    //Aca se imprimen los valores de cada serie
    for jPaso := 1 to NPasosT do
    begin
      trx[kSerie].tr1.PlotNuevo_x(jPaso);
      trx[kSerie].tr1.PlotNuevo_y(iserie_VE, sve.fila(kserie + 1).e(jPaso));
      if kSerie < preview_nseries then
      begin
        if jPasoGuia <= jPasoGuia_Ultimo then
          trx[kSerie].tr1.PlotNuevo_y(iserie_Guia, guia[jPasoGuia]);
        Inc(jPasoGuia);
      end;
      trx[kSerie].tr1.PlotNuevo_y(iserie_pe05, speInf.fila(kserie + 1).e(jPaso));
      trx[kSerie].tr1.PlotNuevo_y(iserie_pe50, spe50.fila(kserie + 1).e(jPaso));
      trx[kSerie].tr1.PlotNuevo_y(iserie_pe95, speSup.fila(kserie + 1).e(jPaso));
    end;

  end;//End del loop que imprime cada serie

  SetLength(colores, 0);
end;

{$ENDIF}


procedure AlFinal;
var
  k: integer;
begin
  {$IFDEF CALIBRAR_PRONOSTICOS_TRAZOSXY}
  (*
  if length( trx ) > 0 then
    for k:= 0 to high( trx ) do
      if trx[k] <> nil then
        trx[k].Close;
        *)
  setlength(trx, 0);
  {$ENDIF}
end;

initialization
  preview_datos := nil;
  preview_fuente := nil;
  {$IFDEF CALIBRAR_PRONOSTICOS_TRAZOSXY}
  setlength(trx, 0);
  {$ENDIF}
finalization
  AlFinal;
end.
