unit ufichasdeterminismos_obsoleta;

(**** 17/11/2011 rch.
La definición de TFichaDeterminismo queda obsoleta pues los determinismos se representan
ahora como parte integral de los pronósticos. (ver unidad upronostico )

Se mantiene la defnición de TFichaDeterminismo solamente para poder leer salas anteriores
a la versión de archivo VERSION = 56.
*****)

interface

uses
  uFuentesAleatorias, MatReal, uCosa, uConversions,
  xMatDefs, udisnormCan, fddp,
  uCosaConNombre, uAuxiliares, uEstados,
  uGlobs, uconstantesSimSEE,
  uFuncionesReales, Classes, umodelosintcegh, uDatosHistoricos,
  uFechas, uFichasLPD, Math;

type

  { TFichaDeterminismo }

  TFichaDeterminismo = class(TCosa)
  public
    valores: TDAofNReal;
    constructor Create(capa: integer; valores: TDAofNReal);
    constructor Create_Default;
    //Crea una TFichaDeterminismo con valores= [0] y corridaDeterminista= false

    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}

    procedure Free; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; override;
  published
    property _valores: TDAofNReal read valores write valores;
    {$ENDIF}

  end;


  TTipoValorEsperadoCEGH = (TTVE_real, TTVE_Gaussiano);
  TDAOfTTipoValorEsperadoCEGH = array of TTipoValorEsperadoCEGH;

  { TFichaFuenteSintetizadorCEGH }

  TFichaFuenteSintetizadorCEGH = class(TFichaLPD)
  public
    modificadoresRelativosVE: boolean;
    multiplicar_vm: boolean;
    modificadoresValEsp, modificadoresDevEst: TDAofNReal;
    deformadores_cache: TMatOf_ddp_VectDeMuestras;

    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      modificadoresRelativos: boolean; multiplicar_VM: boolean;
  modificadoresValEsp, modificadoresDevEst: TDAofNReal);

    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}

    procedure Free; override;
    constructor fichaPorDefecto(nBornesSalida: integer);

    class function DescClase: string; override;
    function infoAd_: string; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataConversionList(): TListDataConversion; override;
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; virtual;
    {$ENDIF}
    procedure AfterInstantiation; override;
  end;


procedure AlInicio;
procedure AlFinal;

implementation

uses
  SysUtils;

constructor TFichaFuenteSintetizadorCEGH.Create(capa: integer; fecha: TFecha;
  periodicidad: TPeriodicidad; modificadoresRelativos: boolean;
  multiplicar_VM: boolean;
  modificadoresValEsp, modificadoresDevEst: TDAofNReal);
begin
  inherited Create(capa, fecha, periodicidad);
  self.modificadoresRelativosVE := modificadoresRelativos;
  self.modificadoresValEsp := modificadoresValEsp;
  self.modificadoresDevEst := modificadoresDevEst;
  self.multiplicar_vm := multiplicar_VM;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TFichaFuenteSintetizadorCEGH.Create_ReadFromText(f: TArchiTexto);
var
  i: integer;

label
  alfinal;

begin

  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  if f.Version < 38 then
    modificadoresRelativosVE := True
  else
    f.rd('modificadoresRelativosVE', modificadoresRelativosVE);

  if f.Version < 31 then
  begin
    multiplicar_vm := False;
  end
  else
    f.rd('multiplicar_vm', multiplicar_vm);

  if f.version >= 27 then
  begin
    f.rd('modificadoresValEsp', modificadoresValEsp);
    f.rd('modificadoresDevEst', modificadoresDevEst);
    f.EjecutarLectura;
    goto alfinal;
  end;

  if f.version >= 23 then
  begin
    f.rd('multiplicadoresVEs', modificadoresValEsp);
    f.rd('multiplicadoresStdDev', modificadoresDevEst);
    f.EjecutarLectura;
    goto alfinal;
  end;

  // if f.version < 23 then
  f.rd('multiplicadoresVEs', modificadoresValEsp);
  f.EjecutarLectura;

  SetLength(modificadoresDevEst, length(modificadoresValEsp));
  for i := 0 to high(modificadoresDevEst) do
    modificadoresDevEst[i] := 1;

  alfinal: ;
end;
{$ENDIF}

procedure TFichaFuenteSintetizadorCEGH.Free;
begin
  modificadoresValEsp := nil;
  modificadoresDevEst := nil;
  if deformadores_cache <> nil then
    freeTMatOf_ddp_VectDeMuestras(deformadores_cache);
  inherited Free;
end;

constructor TFichaFuenteSintetizadorCEGH.fichaPorDefecto(nBornesSalida: integer);
var
  i: integer;
begin
  inherited Create(0, TFecha.Create_Dt(0), nil);
  self.modificadoresRelativosVE := True;
  self.multiplicar_vm := False;
  SetLength(modificadoresValEsp, nBornesSalida);
  SetLength(modificadoresDevEst, nBornesSalida);
  for i := 0 to nBornesSalida - 1 do
  begin
    modificadoresValEsp[i] := 1;
    modificadoresDevEst[i] := 1;
  end;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TFichaFuenteSintetizadorCEGH.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('modificadoresRelativosVE', modificadoresRelativosVE);
  f.wr('multiplicar_vm', multiplicar_vm);
  f.wr('modificadoresValEsp', modificadoresValEsp);
  f.wr('modificadoresDevEst', modificadoresDevEst);
end;
{$ENDIF}


class function TFichaFuenteSintetizadorCEGH.DescClase: string;
begin
  Result := 'Ficha de sintetizador CEGH';
end;

function TFichaFuenteSintetizadorCEGH.infoAd_: string;
var
  i: integer;
  res: string;
begin
  if length(modificadoresValEsp) > 0 then
  begin
    res := res + 'modVE[' + IntToStr(1) + ']= ' +
      FloatToStrF(modificadoresValEsp[0], ffGeneral, 10, 2);
    for i := 1 to high(modificadoresValEsp) do
      res := res + ', modVE[' + IntToStr(i + 1) + ']= ' +
        FloatToStrF(modificadoresValEsp[i], ffGeneral, 10, 2);
  end
  else
    res := '';
  Result := res;
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TFichaFuenteSintetizadorCEGH.CreateDataConversionList: TListDataConversion;
begin
  Result:=inherited CreateDataConversionList();
  Result.Add(27,
             ['multiplicadoresVEs', 'multiplicadoresStdDev'],
             ['modificadoresValEsp', 'modificadoresDevEst'],
             Result.ConversionNAMECHANGE);
end;

class function TFichaFuenteSintetizadorCEGH.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  Result.AddDAOfDoubleColumn('multiplicadoresVEs',              0,  27);
  Result.AddDAOfDoubleColumn('multiplicadoresStdDev',           23, 27);
  Result.AddDAOfDoubleColumn('modificadoresValEsp',             27    );
  Result.AddDAOfDoubleColumn('modificadoresDevEst',             27    );
  Result.AddBooleanColumn   ('modificadoresRelativosVE', true,  38    );
  Result.AddBooleanColumn   ('multiplicar_vm',           false, 31    );
end;
{$ENDIF}

procedure TFichaFuenteSintetizadorCEGH.AfterInstantiation;
var
  i: Integer;
begin
  inherited AfterInstantiation;
  SetLength(modificadoresDevEst, length(modificadoresValEsp));
  for i := 0 to high(modificadoresDevEst) do
    modificadoresDevEst[i] := 1;
end;


constructor TFichaDeterminismo.Create(capa: integer; valores: TDAofNReal);
begin
  inherited Create( capa );
  self.valores := valores;
end;

constructor TFichaDeterminismo.Create_Default;
begin
  inherited Create( 0 );
  SetLength(valores, 1);
  valores[0] := 0;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TFichaDeterminismo.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('valores', valores);
  f.EjecutarLectura;
end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TFichaDeterminismo.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('valores', valores);
end;
{$ENDIF}

procedure TFichaDeterminismo.Free;
begin
  SetLength(valores, 0);
  inherited Free;
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TFichaDeterminismo.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  Result.AddDAOfDoubleColumn('valores');
end;
{$ENDIF}

procedure AlInicio;
begin
  registrarClaseDeCosa(TFichaDeterminismo.ClassName, TFichaDeterminismo);
  registrarClaseDeCosa(TFichaFuenteSintetizadorCEGH.ClassName,
    TFichaFuenteSintetizadorCEGH);
end;

procedure AlFinal;
begin
end;


end.

