unit ufuentesinusoide;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

(*+doc
Implementa una fuente con la función
A * Sin( W * x - FI )
o
A * Cos( W * x - FI )

siendo A, W y FI tres parámetros seleccionables en las fichas dinámicas
al igual que el tipo de función Cos, Sin.

El valor de "x" es tomado del borne de otra fuente.

-doc*)
interface

uses
  uFuentesAleatorias, ucosa, uFichasLPD, xMatDefs, uFechas, SysUtils;

resourcestring
  rsFichaDe = 'Ficha de';
  rsFuenteSinusoide = 'Fuente sinusoide';

type

(*+doc TFichaFuenteSinusoide
Ficha de parámetros dinámicos de las fuente maxmin devuelve la sinusoidal a partir de los parámetros
ingresados
-doc*)

  { TFichaFuenteSinusoide }

  TFichaFuenteSinusoide = class(TFichaLPD)
  public
    (**************************************************************************)
    (* A T R I B U T O S   P E R S I S T E N T E S                            *)
    (**************************************************************************)
    parametroA:   NReal;
    parametroW:   NReal;
    parametroPhi: NReal;
    esCoseno:     boolean;
    fuente:       TFuenteAleatoria;
    borne:        string;
    (**************************************************************************)

    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      a, w, phi: NReal; esCoseno: boolean; fuente: TFuenteAleatoria;
  borne: string);
    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}
    class function DescClase: string; override;
    function infoAd_: string; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; virtual;
    {$ENDIF}
  published
    {$IFDEF MIGRACION_PERSISTENCIA}
    property _parametroA:   NReal            read parametroA   write parametroA;
    property _parametroW:   NReal            read parametroW   write parametroW;
    property _parametroPhi: NReal            read parametroPhi write parametroPhi;
    property _esCoseno:     boolean          read esCoseno     write esCoseno;
    property _fuente:       TFuenteAleatoria read fuente       write fuente;
    property _borne:        string           read borne        write borne;
    {$ENDIF}

  end;

(*+doc TFuenteSinusoide
Da como resultado la sinusoide de los parámetros de entrada
-doc*)
  TFuenteSinusoide = class(TFuenteAleatoriaConFichas)
  private
    //Indices de los bornes de las fuentes de entrada. Deben ser
    //actualizados al cambiar la ficha de parametros dinámicos
    iBorne: integer;

  public
    pa: TFichaFuenteSinusoide;

    constructor Create(capa: integer; nombre: string;
      xdurPasoDeSorteoEnHoras: integer; resumirPromediando: boolean;
  lpd: TFichasLPD);
    class function TipoFichaFuente: TClaseDeFichaLPD; override;
    class function DescClase: string; override;
    procedure RegistrarParametrosDinamicos; override;
    procedure PrepararPaso_ps; override;
    procedure SortearEntradaRB(var aRB: NReal); override;
    procedure ValorEsperadoEntradaRB(var aRB: NReal); override;
  end;

procedure procCambioFichaSinusoide(fuente: TCosa);
procedure AlInicio;
procedure AlFinal;

implementation

//-----------------------------------
// Métodos de TFichaFuenteSinusoide
//===================================

constructor TFichaFuenteSinusoide.Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
  a, w, phi: NReal; esCoseno: boolean; fuente: TFuenteAleatoria; borne: string);

begin
  inherited Create(capa, fecha, periodicidad);
  self.parametroA := a;
  self.parametroW := w;
  self.parametroPhi := phi;
  self.esCoseno := esCoseno;
  self.fuente := fuente;
  self.borne := borne;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TFichaFuenteSinusoide.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('parametroA', parametroA);
  f.rd('parametroW', parametroW);
  f.rd('parametroPhi', parametroPhi);
  f.rd('esCoseno', esCoseno);
  f.rdReferencia('fuente', TCosa(fuente), self);
  f.rd('borne', borne);
  f.EjecutarLectura;
end;

procedure TFichaFuenteSinusoide.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('parametroA', parametroA);
  f.wr('parametroW', parametroW);
  f.wr('parametroPhi', parametroPhi);
  f.wr('esCoseno', esCoseno);
  f.wrReferencia('fuente', TCosa(fuente));
  f.wr('borne', borne);
end;
{$ENDIF}

class function TFichaFuenteSinusoide.DescClase: string;
begin
  Result := rsFichaDe + ' ' + rsFuenteSinusoide;
end;

function TFichaFuenteSinusoide.infoAd_: string;
begin
  Result := 'parametroA= ' + FloatToStrF(parametroA, ffGeneral, 10, 1) +
    ',' + 'parametroW= ' + FloatToStrF(parametroW, ffGeneral, 10, 1) +
    ',' + 'parametroPhi=' + FloatToStrF(parametroPhi, ffGeneral, 10, 1) +
    ',' + 'esCoseno=' + BoolToStr(esCoseno) + ',' + 'fuente= ' +
    fuente.nombre + ', ' + 'borne= ' + borne;
  ;
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TFichaFuenteSinusoide.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  Result.AddDoubleColumn ('parametroA',   0.0 );
  Result.AddDoubleColumn ('parametroW',   0.0 );
  Result.AddDoubleColumn ('parametroPhi', 0.0 );
  Result.AddBooleanColumn('esCoseno',     true);
  Result.AddCosaColumn   ('fuente'            );
  Result.AddStringColumn ('borne',        ''  );
end;
{$ENDIF}

//------------------------------
// Métodos de TFuenteSinusoide
//==============================

constructor TFuenteSinusoide.Create(capa: integer; nombre: string; xdurPasoDeSorteoEnHoras: integer;
  resumirPromediando: boolean; lpd: TFichasLPD);
begin
  inherited Create(capa, nombre, xdurPasoDeSorteoEnHoras, resumirPromediando, lpd);
end;

class function TFuenteSinusoide.TipoFichaFuente: TClaseDeFichaLPD;
begin
  Result := TFichaFuenteSinusoide;
end;

{function TFuenteTiempo.InfoAd : String;
begin
  result:= 'Tiempo';
end;}

class function TFuenteSinusoide.DescClase: string;
begin
  Result := rsFuenteSinusoide;
end;

procedure TFuenteSinusoide.RegistrarParametrosDinamicos;
begin
  //Se llama solo despues de preparar memoria que ya expande las fichas
  //no hace falta volver a hacerlo y es un proceso caro si hay periodicidad
  //   lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pa,
    nil, procCambioFichaSinusoide);
end;

procedure TFuenteSinusoide.PrepararPaso_ps;
begin
  if (pa.esCoseno) then
    bornera[0] := pa.parametroA * cos(pa.parametroW * pa.fuente.Bornera[iBorne] -
      pa.parametroPhi)
  else
    bornera[0] := pa.parametroA * sin(pa.parametroW * pa.fuente.Bornera[iBorne] -
      pa.parametroPhi);
end;

procedure TFuenteSinusoide.SortearEntradaRB(var aRB: NReal);
begin
  aRB := 0;
end;

procedure TFuenteSinusoide.ValorEsperadoEntradaRB(var aRB: NReal);
begin
  aRB := 0;
end;


procedure procCambioFichaSinusoide(fuente: TCosa);
begin
  cambioFichaPDFuenteAleatoria(fuente);
  TFuenteSinusoide(fuente).iBorne :=
    TFuenteSinusoide(fuente).pa.fuente.IdBorne(TFuenteSinusoide(fuente).pa.borne);
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TFuenteSinusoide.ClassName, TFuenteSinusoide);
  registrarClaseDeCosa(TFichaFuenteSinusoide.ClassName, TFichaFuenteSinusoide);
end;

procedure AlFinal;
begin
end;

end.
